classdef FuncGenInterface < InstrumentInterface & matlab.mixin.Heterogeneous
    %FUNCGENINTERFACE Interface class for function generators
    %   All function generators derive from this class which allows for a
    %   heterogeneous array of function generators in the measurement setup 
    %   class.
    
    properties (Constant)
        INSTRUMENT_TYPE = "FUNCGEN"
    end
    
    properties (Constant, Abstract)
        NUM_CHANNELS
    end
    
    methods
        function obj = FuncGenInterface(portName, baudRate, logHandle)
            arguments
                portName (1,1) string
                baudRate (1,1) double
                logHandle (1,1) Logger
            end
            %FUNCGENINTERFACE Construct an instance of this class
            %   Call InstrumentInterface constructor with FUNCGEN type.
            obj = obj@InstrumentInterface(...
                SupplyInterface.INSTRUMENT_TYPE, ...
                portName, ...
                baudRate, ...
                logHandle);

            obj.Port.writeline("*RST")

            % Instrument specific setup
            obj.log("Instrument setup done")
        end

        function delete(obj)
            %DELETE Destruct class instance
            %   Reset the instrument to ensure that all outputs are off and
            %   everything is reset.
            obj.Port.writeline("*RST")
        end
    end

    methods (Sealed)
        function numChannels = getNumChannels(obj)
            %GETNUMCHANNELS Return number of channels of funcgen object
            numChannels = obj.NUM_CHANNELS;
        end
    end
        
    methods (Abstract)
        setChannelState(obj, channel, state)
        setChannelWaveform(obj, channel, waveform)
        setChannelFrequency(obj, channel, frequency)
        setChannelVoltageRange(obj, channel, voltageRange)
    end
end

