classdef RelayInterface < InstrumentInterface
    %RELAYINTERFACE Summary of this class goes here
    %   Detailed explanation goes here

    properties (Constant)
        INSTRUMENT_TYPE = "RELAY"
        SET_GROUPS = {[1,2,3,4]} % Dependency: set only one of each group
    end
    
    properties

    end
    
    methods
        function obj = RelayInterface(serialPort, baudRate, logHandle)
            arguments
                serialPort (1,1) string
                baudRate (1,1) double
                logHandle (1,1) Logger
            end
            %RELAYINTERFACE Construct an instance of this class
            %   Call InstrumentInterface constructor with RELAY type.
            obj = obj@InstrumentInterface(...
                RelayInterface.INSTRUMENT_TYPE, ...
                serialPort, ...
                baudRate, ...
                logHandle);

            % Instrument specific setup
            obj.log("Instrument setup done")
        end
        
        function setRelay(obj, channel)
            arguments
                obj (1,1) RelayInterface
                channel (1,1) uint8 {mustBeInRange(channel,1,7)}
            end
            %SETRELAY Set relay on specified channel
            % Find if any set group contains the selected channel
            setGroups = cellfun(@(x)(any(x == channel)), obj.SET_GROUPS);
            % Reset all other channels
            resetChannels = unique([obj.SET_GROUPS{setGroups}]);
            for resetChannel = resetChannels
                if resetChannel ~= channel
                    obj.resetRelay(resetChannel)
                end
            end
            obj.Port.write(channel + 9, "uint8")
            obj.log("Relay %d set", channel)
        end
        
        function resetRelay(obj, channel)
            arguments
                obj (1,1) RelayInterface
                channel (1,1) uint8 {mustBeInRange(channel,1,7)}
            end
            %RESETRELAY Reset relay on specified channel
            obj.Port.write(channel - 1, "uint8")
            obj.log("Relay %d reset", channel)
        end

        function waitForDriver(obj)
            arguments
                obj (1,1) RelayInterface
            end
            %WAITFORRELAY Block until relay driver responds and flush
            %buffer afterwards
            % Wait for something in the read buffer
            obj.Port.read(1, "uint8");
            % Flush the input buffer to prevent accumulation of old data
            obj.Port.flush("input")
            obj.log("Relay driver responded")
        end
    end
end

