classdef SmuInterface < InstrumentInterface & matlab.mixin.Heterogeneous
    %SMUINTERFACE Interface class for SMUs
    %   All SMUs derive from this class which allows for a heterogeneous
    %   array of SMUs in the measurement setup class.

    properties (Constant)
        INSTRUMENT_TYPE = "SMU"
    end
    
    properties (Constant, Abstract)
        NUM_CHANNELS
    end
    
    methods
        function obj = SmuInterface(serialPort, baudRate, logHandle)
            arguments
                serialPort (1,1) string
                baudRate (1,1) double
                logHandle (1,1) Logger
            end
            %SMUINTERFACE Construct an instance of this class
            %   Call InstrumentInterface constructor with SMU type.
            obj = obj@InstrumentInterface(...
                SmuInterface.INSTRUMENT_TYPE, ...
                serialPort, ...
                baudRate, ...
                logHandle);

            obj.Port.writeline("*RST")

            % Instrument specific setup
            obj.log("Instrument setup done")
        end

        function delete(obj)
            %DELETE Destruct class instance
            %   Reset the instrument to ensure that all outputs are off and
            %   everything is reset.
            obj.Port.writeline("*RST")
        end
    end

    methods (Sealed)
        function numChannels = getNumChannels(obj)
            %GETNUMCHANNELS Return number of channels of supply object
            numChannels = obj.NUM_CHANNELS;
        end
    end
        
    methods (Abstract)
        setChannelSourceSettling(obj, channel, settlingTime)
        setChannelNplc(obj, channel, nplc)
        setChannelSourceState(obj, channel, state)
        setChannelSourceVoltage(obj, channel, voltage)
        setChannelSourceCurrent(obj, channel, current)
        setChannelRangeVoltage(obj, channel, voltage)
        setChannelRangeCurrent(obj, channel, current)
        setChannelLimitVoltage(obj, channel, voltage)
        setChannelLimitCurrent(obj, channel, current)
        setChannelSense(obj, channel, mode)
        setChannelMode(obj, channel, mode)
        setChannelDisplay(obj, channel, mode)
        setChannelHighCap(obj, channel, mode)
        voltage = getChannelVoltage(obj, channel)
        current = getChannelCurrent(obj, channel)
        voltage = measureChannelVoltage(obj, channel)
        power = measureChannelPower(obj, channel)
        current = measureChannelCurrent(obj, channel)
    end
end

