classdef SupplyInterface < InstrumentInterface & matlab.mixin.Heterogeneous
    %SUPPLYINTERFACE Interface class for supplies
    %   All supplies derive from this class which allows for a
    %   heterogeneous array of supplies in the measurement setup class.

    properties (Constant)
        INSTRUMENT_TYPE = "SUPPLY"
    end
    
    properties (Constant, Abstract)
        NUM_CHANNELS
        CHANNEL_VOLTAGE_RANGE
    end
    
    methods
        function obj = SupplyInterface(portName, baudRate, logHandle)
            arguments
                portName (1,1) string
                baudRate (1,1) double
                logHandle (1,1) Logger
            end
            %SUPPLYINTERFACE Construct an instance of this class
            %   Call InstrumentInterface constructor with SUPPLY type.
            obj = obj@InstrumentInterface(...
                SupplyInterface.INSTRUMENT_TYPE, ...
                portName, ...
                baudRate, ...
                logHandle);

            obj.Port.writeline("*RST")

            % Instrument specific setup
            obj.log("Instrument setup done")
        end

        function delete(obj)
            %DELETE Destruct class instance
            %   Reset the instrument to ensure that all outputs are off and
            %   everything is reset.
            obj.Port.writeline("*RST")
        end
    end

    methods (Sealed)
        function numChannels = getNumChannels(obj)
            %GETNUMCHANNELS Return number of channels of supply object
            numChannels = obj.NUM_CHANNELS;
        end
    end

    methods (Abstract)
        setChannelState(obj, channel, state)
        setChannelVoltage(obj, channel, voltage)
        setChannelCurrent(obj, channel, current)
        voltage = getChannelVoltage(obj, channel)
        current = getChannelCurrent(obj, channel)
        voltage = measureChannelVoltage(obj, channel)
        current = measureChannelCurrent(obj, channel)
    end
end

