charMemsWorkspaceNames = ["charMems_20230406172956365.mat", ...
    "charMems_20230407111710500.mat", "charMems_20230407124013212.mat"];

retTimeWorkspaceNames = ["test6_mult_20230411090830355.mat"];

readEnergy = zeros(32,numel(charMemsWorkspaceNames));
writeEnergy = zeros(32,numel(charMemsWorkspaceNames));
refreshEnergy = zeros(32,numel(charMemsWorkspaceNames));
leakagePower = zeros(32,numel(charMemsWorkspaceNames));
latency = zeros(32,numel(charMemsWorkspaceNames));

for iMeas = 1:numel(charMemsWorkspaceNames)
    load(charMemsWorkspaceNames(iMeas))
    readEnergy(:,iMeas) = memoryCharacterisationOutputData(:,2);
    writeEnergy(:,iMeas) = memoryCharacterisationOutputData(:,3);
    refreshEnergy(:,iMeas) = memoryCharacterisationOutputData(:,4);
    leakagePower(:,iMeas) = memoryCharacterisationOutputData(:,5);
    latency(:,iMeas) = memoryCharacterisationOutputData(:,6);
end

% RO speedup correction (latency is calculated using the 4K RO speed instead of RT)
latency = (latency - 180e-12)/17*20 + 180e-12;

memoryNumbers = inputData(:,1);
references = inputData(:,5);
leakagePower(memoryNumbers < 6,:) = leakagePower(memoryNumbers < 6,:) / 6;

retTime = inputData(:,6);
retTimeCount = zeros(32,1);
retTimeMu = Inf(32,1);
retTimeSigma = zeros(32,1);

fcn = @(b,x) normcdf(x, b(1), b(2));
% Add actual retention times from the more elaborate measurement script
for iRet=1:numel(retTimeWorkspaceNames)
    load(retTimeWorkspaceNames(iRet))
    for memoryIdx = 1:numel(memoryNo)
	    retTimeTemp(rowNo+1,:) = 8e-8 + 2e-8 * reshape(min(criticalDelay( ...
        memoryIdx,:,:,1,:), [], 3), [numel(rowNo), 32]);
	    NRCF = @(b) norm(sum(retTimeTemp(retTimeTemp ~= Inf)' <= retTimeTemp( ...
		    retTimeTemp ~= Inf), 2)/numel(retTimeTemp) - fcn(b, log(retTimeTemp( ...
		    retTimeTemp ~= Inf))));
	    B = fminsearch(NRCF, [-5; 1]);
        worstCellRetTime = min(retTimeTemp, [], 'all');
        
        select = and(memoryNumbers == memoryNo(memoryIdx), ...
            references == memoryReferenceVoltages(memoryIdx));
        retTime(select) = worstCellRetTime;
        retTimeCount(select) = sum(retTimeTemp == worstCellRetTime, 'all');
        retTimeMu(select) = B(1);
        retTimeSigma(select) = B(2);
        
%         fprintf("%2d: mu, sigma, min = %7.3f, %7.3f, %e (%dx)\n", ...
% 		    memoryIdx, B(1), B(2), worstCellRetTime(memoryIdx), ...
% 		    sum(retTime == min(retTime, [], 'all'), 'all'))
    end
end

% Build application space figure with min and max energy values
% Compute power over application space for each memory type
writeFrequencyRatio = linspace(0, 0.5, 101);
operationFrequency = logspace(log10(1e3), log10(1e9), 501);

[oF, wFR] = meshgrid(operationFrequency, writeFrequencyRatio);

readFreq = oF .* (1 - wFR);
writeFreq = oF .* wFR;

power = NaN([size(oF), numel(memoryNumbers), 3]);

for iMemory = 1:numel(memoryNumbers)
    refreshFreq = 1/retTime(iMemory);
    % Min
    powerTemp = readFreq * min(readEnergy(iMemory, :)) + ...
        writeFreq * min(writeEnergy(iMemory, :)) + ...
        refreshFreq * min(refreshEnergy(iMemory, :)) + ...
        max(min(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,1) = powerTemp;
    % Max
    powerTemp = readFreq * max(readEnergy(iMemory, :)) + ...
        writeFreq * max(writeEnergy(iMemory, :)) + ...
        refreshFreq * max(refreshEnergy(iMemory, :)) + ...
        max(max(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,2) = powerTemp;
    % Mean
    powerTemp = readFreq * mean(readEnergy(iMemory, :)) + ...
        writeFreq * mean(writeEnergy(iMemory, :)) + ...
        refreshFreq * mean(refreshEnergy(iMemory, :)) + ...
        max(mean(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,3) = powerTemp;
end

selPower = zeros(numel(memoryNumbers), 1);
minPower = min(power(:,:,:,3), [], 3);
for iMemory = 1:numel(memoryNumbers)
    memPower = power(:,:,iMemory, 3);
    selPower(iMemory) = any(memPower == minPower, 'all');
end

selMemoryWise = zeros(numel(memoryNumbers), 1);
% for iMemory = 1:numel(memoryNumbers)
%     memPower = power(:,:,iMemory, 3);
%     memNumber = memoryNumbers(iMemory);
%     minPower = min(power(:,:,memoryNumbers == memNumber,3), [], 3);
%     selMemoryWise(iMemory) = any(memPower == minPower, 'all');
% end
for iMemory = 1:numel(memoryNumbers)
    if ~any(memoryNumbers(selPower == 1) == memoryNumbers(iMemory))
        %memPower = power(:,:,iMemory, 3);
        memNumber = memoryNumbers(iMemory);
        %minPower = min(power(:,:,memoryNumbers == memNumber,3), [], 3);
        %selMemoryWise(iMemory) = any(memPower == minPower, 'all');

        % Max retention time
        selMemoryWise(iMemory) = (retTime(iMemory) == ...
            max(retTime(memoryNumbers == memNumber)));        
    end
end

% Select only the memory configurations that have the lowest power 
% somewhere for each memory type
selection = and(or(selPower, selMemoryWise), ~any(latency > 3e-9, 2));

%% Application space figure
colorLimits = [...
    1.0, 1.0, 0.5, 0.5, 0.0, 0.0;
    1.0, 1.0, 0.0, 0.0, 0.0, 0.0;
    0.7, 0.7, 1.0, 1.0, 0, 0;
    0.0, 0.0, 1.0, 1.0, 0, 0;
    0.0, 0.0, 0.0, 0.0, 1.0, 1.0;
    0.0, 0.0, 0.7, 0.7, 1.0, 1.0;
    0.4, 0.4, 0.4, 0.4, 0.4, 0.4;
    0.7, 0.7, 0.7, 0.7, 0.7, 0.7
    ];

dataColors = [];
for memoryNumber = 0:7
    count = sum(memoryNumbers == memoryNumber);
    if count > 0
        tmp = [linspace(colorLimits(memoryNumber + 1, 1), ...
            colorLimits(memoryNumber + 1, 2), count)', ...
            linspace(colorLimits(memoryNumber + 1, 3), ...
            colorLimits(memoryNumber + 1, 4), count)', ...
            linspace(colorLimits(memoryNumber + 1, 5), ...
            colorLimits(memoryNumber + 1, 6), count)'];
        dataColors = [dataColors; tmp];
    end
end

% Plot
figure(12)
%t = tiledlayout(1, 2, "TileSpacing", "tight", "Padding", "tight");
sub_handle(1) = subplot(1,2,1);
applicationSpace = zeros(size(oF));
minPower = min(power(:,:,selection,3), [], 3);
for iMemory = 1:numel(memoryNumbers)
    applicationSpace(power(:,:,iMemory,3) == minPower) = iMemory;
end
hold on
for i=1:8
    fill([0,1,0], [0,0,1], colorLimits(i, 1:2:end))
end
% img = imagesc(operationFrequency, writeFrequencyRatio./(1 - writeFrequencyRatio), applicationSpace);
contourf(operationFrequency, writeFrequencyRatio./(1 - writeFrequencyRatio), applicationSpace, size(dataColors, 1), 'EdgeColor', 'none');
hold off
% img.Parent.YDir = 'normal';
% img.Parent.XScale = 'log';
ax = gca;
ax.YDir = "normal";
ax.XScale = "log";
xlim([min(operationFrequency), max(operationFrequency)])
%xlabel("Operation frequency [Hz]")
ylabel("W/R")
ylim([0, 1])
sub_handle(1).YTick = [0,0.25,0.5,0.75,1];
sub_handle(1).XTick = 10.^(3:2:9);
% colormap(img.Parent, [1, 1, 1; dataColors])
colormap(ax.Parent, [1, 1, 1; dataColors])
clim([0, size(dataColors,1)])
title("RT")
box on
% xticklabels(ax1, {})
legend('2T NW-PR (SVT)', '2T NW-PR (LVT)', '3T NW-PR (SVT)',  ...
    '3T NW-PR (LVT)', '3T PW-PR (SVT)',  '3T PW-PR (LVT)', ...
    '6T SRAM (SVT)', '6T SRAM (LVT)', 'Location', 'westoutside')

%% Print table
for memoryNo = 0:7
    fprintf("----------------------------------------------------------\n")
    fprintf("Memory %d\n", memoryNo)
    % Retention time
    fprintf("Retention time:\n")
    mu = retTimeMu(and(selection, memoryNumbers == memoryNo));
    fprintf("Mu: %.3f - %.3f\n", min(mu), max(mu))
    sigma = retTimeSigma(and(selection, memoryNumbers == memoryNo));
    fprintf("Sigma: %.3f - %.3f\n", min(sigma), max(sigma))
    worstTime = retTime(and(selection, memoryNumbers == memoryNo));
    worstAmount = retTimeCount(and(selection, memoryNumbers == memoryNo));
    [~,iMin] = min(worstTime);
    [~,iMax] = max(worstTime);
    fprintf("Worst: %e (%dx) - %e (%dx)\n", min(worstTime), ...
        worstAmount(iMin), max(worstTime), worstAmount(iMax))
    % Power
    fprintf("Energy/power:\n")
    Er = readEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Eread: %.3f - %.3f fJ/op\n", 1e15 * min(Er, [], "all"), ...
        1e15 * max(Er, [], "all"))
    Ew = writeEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Ewrite: %.3f - %.3f fJ/op\n", 1e15 * min(Ew, [], "all"), ...
        1e15 * max(Ew, [], "all"))
    Eref = refreshEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Erefresh: %.3f - %.3f pJ\n", 1e12 * min(Eref, [], "all"), ...
        1e12 * max(Eref, [], "all"))
    Tlat = latency(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Latency: %.3f - %.3f ns\n", 1e9 * min(Tlat, [], "all"), ...
        1e9 * max(Tlat, [], "all"))
end

%% 4K

charMemsWorkspaceNames = ["charMems_20230405194241039.mat", ...
    "charMems_20230406111705001.mat", "charMems_20230406125754577.mat"];

retTimeWorkspaceNames = ["test6_mult_20230405221043008.mat", "test6_mult_20230406082430754.mat"];

readEnergy = zeros(32,numel(charMemsWorkspaceNames));
writeEnergy = zeros(32,numel(charMemsWorkspaceNames));
refreshEnergy = zeros(32,numel(charMemsWorkspaceNames));
leakagePower = zeros(32,numel(charMemsWorkspaceNames));
latency = zeros(32,numel(charMemsWorkspaceNames));

for iMeas = 1:numel(charMemsWorkspaceNames)
    load(charMemsWorkspaceNames(iMeas))
    readEnergy(:,iMeas) = memoryCharacterisationOutputData(:,2);
    writeEnergy(:,iMeas) = memoryCharacterisationOutputData(:,3);
    refreshEnergy(:,iMeas) = memoryCharacterisationOutputData(:,4);
    leakagePower(:,iMeas) = memoryCharacterisationOutputData(:,5);
    latency(:,iMeas) = memoryCharacterisationOutputData(:,6);
end

memoryNumbers = inputData(:,1);
references = inputData(:,5);
leakagePower(memoryNumbers < 6,:) = leakagePower(memoryNumbers < 6,:) / 6;

retTime = inputData(:,6);
retTimeCount = zeros(32,1);
retTimeMu = Inf(32,1);
retTimeSigma = zeros(32,1);
    
% Add actual retention times from the more elaborate measurement script
for iRet=1:numel(retTimeWorkspaceNames)
    load(retTimeWorkspaceNames(iRet))
    for memoryIdx = 1:numel(memoryNo)
	    retTimeTemp(rowNo+1,:) = 8e-8 + 2e-8 * reshape(min(criticalDelay( ...
        memoryIdx,:,:,1,:), [], 3), [numel(rowNo), 32]);
	    NRCF = @(b) norm(sum(retTimeTemp(retTimeTemp ~= Inf)' <= retTimeTemp( ...
		    retTimeTemp ~= Inf), 2)/numel(retTimeTemp) - fcn(b, log(retTimeTemp( ...
		    retTimeTemp ~= Inf))));
	    B = fminsearch(NRCF, [-5; 1]);
        worstCellRetTime = min(retTimeTemp, [], 'all');
        
        select = and(memoryNumbers == memoryNo(memoryIdx), ...
            references == memoryReferenceVoltages(memoryIdx));
        retTime(select) = worstCellRetTime;
        retTimeCount(select) = sum(retTimeTemp == worstCellRetTime, 'all');
        retTimeMu(select) = B(1);
        retTimeSigma(select) = B(2);
        
%         fprintf("%2d: mu, sigma, min = %7.3f, %7.3f, %e (%dx)\n", ...
% 		    memoryIdx, B(1), B(2), worstCellRetTime(memoryIdx), ...
% 		    sum(retTime == min(retTime, [], 'all'), 'all'))
    end
end

% Build application space figure with min and max energy values
% Compute power over application space for each memory type
writeFrequencyRatio = linspace(0, 0.5, 101);
operationFrequency = logspace(log10(1e3), log10(1e9), 501);

[oF, wFR] = meshgrid(operationFrequency, writeFrequencyRatio);

readFreq = oF .* (1 - wFR);
writeFreq = oF .* wFR;

power = NaN([size(oF), numel(memoryNumbers), 3]);

for iMemory = 1:numel(memoryNumbers)
    refreshFreq = 1/retTime(iMemory);
    % Min
    powerTemp = readFreq * min(readEnergy(iMemory, :)) + ...
        writeFreq * min(writeEnergy(iMemory, :)) + ...
        refreshFreq * min(refreshEnergy(iMemory, :)) + ...
        max(min(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,1) = powerTemp;
    % Max
    powerTemp = readFreq * max(readEnergy(iMemory, :)) + ...
        writeFreq * max(writeEnergy(iMemory, :)) + ...
        refreshFreq * max(refreshEnergy(iMemory, :)) + ...
        max(max(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,2) = powerTemp;
    % Mean
    powerTemp = readFreq * mean(readEnergy(iMemory, :)) + ...
        writeFreq * mean(writeEnergy(iMemory, :)) + ...
        refreshFreq * mean(refreshEnergy(iMemory, :)) + ...
        max(mean(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,3) = powerTemp;
end

selPower = zeros(numel(memoryNumbers), 1);
minPower = min(power(:,:,:,3), [], 3);
for iMemory = 1:numel(memoryNumbers)
    memPower = power(:,:,iMemory, 3);
    selPower(iMemory) = any(memPower == minPower, 'all');
end

selMemoryWise = zeros(numel(memoryNumbers), 1);
% for iMemory = 1:numel(memoryNumbers)
%     memPower = power(:,:,iMemory, 3);
%     memNumber = memoryNumbers(iMemory);
%     minPower = min(power(:,:,memoryNumbers == memNumber,3), [], 3);
%     selMemoryWise(iMemory) = any(memPower == minPower, 'all');
% end
for iMemory = 1:numel(memoryNumbers)
    if ~any(memoryNumbers(selPower == 1) == memoryNumbers(iMemory))
        %memPower = power(:,:,iMemory, 3);
        memNumber = memoryNumbers(iMemory);
        %minPower = min(power(:,:,memoryNumbers == memNumber,3), [], 3);
        %selMemoryWise(iMemory) = any(memPower == minPower, 'all');

        % Max retention time
        selMemoryWise(iMemory) = (retTime(iMemory) == ...
            max(retTime(memoryNumbers == memNumber)));        
    end
end

% Select only the memory configurations that have the lowest power 
% somewhere for each memory type
selection = and(or(selPower, selMemoryWise), ~any(latency > 3e-9, 2));

%% Application space figure
dataColors = [];
for memoryNumber = 0:7
    count = sum(memoryNumbers == memoryNumber);
    if count > 0
        tmp = [linspace(colorLimits(memoryNumber + 1, 1), ...
            colorLimits(memoryNumber + 1, 2), count)', ...
            linspace(colorLimits(memoryNumber + 1, 3), ...
            colorLimits(memoryNumber + 1, 4), count)', ...
            linspace(colorLimits(memoryNumber + 1, 5), ...
            colorLimits(memoryNumber + 1, 6), count)'];
        dataColors = [dataColors; tmp];
    end
end

% Plot
figure(12)
% t = tiledlayout(10,1);
sub_handle(2) = subplot(1,2,2);
applicationSpace = zeros(size(oF));
minPower = min(power(:,:,selection,3), [], 3);
for iMemory = 1:numel(memoryNumbers)
    applicationSpace(power(:,:,iMemory,3) == minPower) = iMemory;
end
hold on
for i=1:8
    fill([0,1,0], [0,0,1], colorLimits(i, 1:2:end))
end
% img = imagesc(operationFrequency, writeFrequencyRatio./(1 - writeFrequencyRatio), applicationSpace);
contourf(operationFrequency, writeFrequencyRatio./(1 - writeFrequencyRatio), applicationSpace, size(dataColors, 1), 'EdgeColor', 'none');
hold off
% img.Parent.YDir = 'normal';
% img.Parent.XScale = 'log';
ax = gca;
ax.YDir = "normal";
ax.XScale = "log";
xlim([min(operationFrequency), max(operationFrequency)])
%ylabel("f_W/(f_R + f_W)")
ylim([0, 1])
sub_handle(2).YTick = [0,0.25,0.5,0.75,1];
sub_handle(2).XTick = 10.^(3:2:9);
% ax3.YTickLabel = {};
% colormap(img.Parent, [1, 1, 1; dataColors])
colormap(ax, [1, 1, 1; dataColors])
clim([0, size(dataColors,1)])
title("4.2K")
box on
%xlabel("Operation frequency [Hz]")
% xticklabels(ax3, {})
% legend('2T NW-PR (SVT)', '2T NW-PR (LVT)', '3T NW-PR (SVT)',  ...
%     '3T NW-PR (LVT)', '3T PW-PR (SVT)',  '3T PW-PR (LVT)', ...
%     '6T SRAM (SVT)', '6T SRAM (LVT)', 'Location', 'southoutside', ...
%     'NumColumns', 2)

%% Print table
for memoryNo = 0:7
    fprintf("----------------------------------------------------------\n")
    fprintf("Memory %d\n", memoryNo)
    % Retention time
    fprintf("Retention time:\n")
    mu = retTimeMu(and(selection, memoryNumbers == memoryNo));
    fprintf("Mu: %.3f - %.3f\n", min(mu), max(mu))
    sigma = retTimeSigma(and(selection, memoryNumbers == memoryNo));
    fprintf("Sigma: %.3f - %.3f\n", min(sigma), max(sigma))
    worstTime = retTime(and(selection, memoryNumbers == memoryNo));
    worstAmount = retTimeCount(and(selection, memoryNumbers == memoryNo));
    [~,iMin] = min(worstTime);
    [~,iMax] = max(worstTime);
    fprintf("Worst: %e (%dx) - %e (%dx)\n", min(worstTime), ...
        worstAmount(iMin), max(worstTime), worstAmount(iMax))
    % Power
    fprintf("Energy/power:\n")
    Er = readEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Eread: %.3f - %.3f fJ/op\n", 1e15 * min(Er, [], "all"), ...
        1e15 * max(Er, [], "all"))
    Ew = writeEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Ewrite: %.3f - %.3f fJ/op\n", 1e15 * min(Ew, [], "all"), ...
        1e15 * max(Ew, [], "all"))
    Eref = refreshEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Erefresh: %.3f - %.3f pJ\n", 1e12 * min(Eref, [], "all"), ...
        1e12 * max(Eref, [], "all"))
    Tlat = latency(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Latency: %.3f - %.3f ns\n", 1e9 * min(Tlat, [], "all"), ...
        1e9 * max(Tlat, [], "all"))
end

%% Figure sizing
fig_handle = figure(12);
fig_width  = 8.6; %cm
fig_height = 3.1; %cm
set(fig_handle, "Units", "centimeters")
set(fig_handle, 'Position',[60 25 fig_width fig_height]);
%set(fig_handle, 'Position',[-20 0 fig_width fig_height]);
set(fig_handle.Children(2), "ItemTokenSize", [10,18])

set(fig_handle, 'PaperUnits', 'centimeters')
set(fig_handle, 'PaperSize', [fig_width, fig_height]*1.01)

subs_xoff = 0.40;
sub_xoff = 0.02;
sub_yoff = 0.20;
sub_width = 0.24;
sub_height = 0.69;
for sub_idx = 1:2
    set(sub_handle(sub_idx), "FontName", "Times")
    set(sub_handle(sub_idx), "FontSize", 7)
    set(sub_handle(sub_idx), "Position", [subs_xoff + ((1-subs_xoff)/2 + sub_xoff) * (sub_idx - 1), sub_yoff, sub_width, sub_height])
end
% Shared xlabel annotation
xlabel_yoff = -0.18;
annotation("textbox", [subs_xoff, sub_yoff + xlabel_yoff, ((1-subs_xoff)/2 + sub_xoff) + sub_width, 0.1], "String", "Access rate [Hz]", "HorizontalAlignment", "center", "FontName", "Times", "FontSize", 7, "LineStyle", "none")

% set(fig_handle, 'Renderer', 'painters')
% saveas(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\power_comparison.svg", 'svg')
% exportgraphics(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\power_comparison.eps", "ContentType", "vector")
% print(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\power_comparison.pdf", "-vector")