charMemsWorkspaceNames = ["charMems_20230406172956365.mat", ...
    "charMems_20230407111710500.mat", "charMems_20230407124013212.mat"];

retTimeWorkspaceNames = ["test6_mult_20230411090830355.mat"];

readEnergy = zeros(32,numel(charMemsWorkspaceNames));
writeEnergy = zeros(32,numel(charMemsWorkspaceNames));
refreshEnergy = zeros(32,numel(charMemsWorkspaceNames));
leakagePower = zeros(32,numel(charMemsWorkspaceNames));
latency = zeros(32,numel(charMemsWorkspaceNames));

for iMeas = 1:numel(charMemsWorkspaceNames)
    load(charMemsWorkspaceNames(iMeas))
    readEnergy(:,iMeas) = memoryCharacterisationOutputData(:,2);
    writeEnergy(:,iMeas) = memoryCharacterisationOutputData(:,3);
    refreshEnergy(:,iMeas) = memoryCharacterisationOutputData(:,4);
    leakagePower(:,iMeas) = memoryCharacterisationOutputData(:,5);
    latency(:,iMeas) = memoryCharacterisationOutputData(:,6);
end

% RO speedup correction (latency is using the 4K RO speed instead of RT)
latency = (latency - 180e-12)/17*20 + 180e-12;

memoryNumbers = inputData(:,1);
references = inputData(:,5);
leakagePower(memoryNumbers < 6,:) = leakagePower(memoryNumbers < 6,:) / 6;

retTime = inputData(:,6);
retTimeCount = zeros(32,1);
retTimeMu = Inf(32,1);
retTimeSigma = zeros(32,1);

fcn = @(b,x) normcdf(x, b(1), b(2));
% Add actual retention times from the more elaborate measurement script
for iRet=1:numel(retTimeWorkspaceNames)
    load(retTimeWorkspaceNames(iRet))
    for memoryIdx = 1:numel(memoryNo)
	    retTimeTemp(rowNo+1,:) = 8e-8 + 2e-8 * reshape(min(criticalDelay( ...
        memoryIdx,:,:,1,:), [], 3), [numel(rowNo), 32]);
	    NRCF = @(b) norm(sum(retTimeTemp(retTimeTemp ~= Inf)' <= retTimeTemp( ...
		    retTimeTemp ~= Inf), 2)/numel(retTimeTemp) - fcn(b, log(retTimeTemp( ...
		    retTimeTemp ~= Inf))));
	    B = fminsearch(NRCF, [-5; 1]);
        worstCellRetTime = min(retTimeTemp, [], 'all');
        
        select = and(memoryNumbers == memoryNo(memoryIdx), ...
            references == memoryReferenceVoltages(memoryIdx));
        retTime(select) = worstCellRetTime;
        retTimeCount(select) = sum(retTimeTemp == worstCellRetTime, 'all');
        retTimeMu(select) = B(1);
        retTimeSigma(select) = B(2);
        
%         fprintf("%2d: mu, sigma, min = %7.3f, %7.3f, %e (%dx)\n", ...
% 		    memoryIdx, B(1), B(2), worstCellRetTime(memoryIdx), ...
% 		    sum(retTime == min(retTime, [], 'all'), 'all'))
    end
end

% Build application space figure with min and max energy values
% Compute power over application space for each memory type
writeReadRatio = linspace(0, 1, 2);
operationFrequency = logspace(log10(1e3), log10(1e9), 501);

[oF, wRR] = meshgrid(operationFrequency, writeReadRatio);

readFreq = oF ./ (1 + wRR);
writeFreq = oF - readFreq;

powerrt = NaN([size(oF), numel(memoryNumbers), 3]);

for iMemory = 1:numel(memoryNumbers)
    refreshFreq = 1/retTime(iMemory);
    % Min
    powerTemp = readFreq * min(readEnergy(iMemory, :)) + ...
        writeFreq * min(writeEnergy(iMemory, :)) + ...
        refreshFreq * min(refreshEnergy(iMemory, :)) + ...
        max(min(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    powerrt(:,:,iMemory,1) = powerTemp;
    % Max
    powerTemp = readFreq * max(readEnergy(iMemory, :)) + ...
        writeFreq * max(writeEnergy(iMemory, :)) + ...
        refreshFreq * max(refreshEnergy(iMemory, :)) + ...
        max(max(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    powerrt(:,:,iMemory,2) = powerTemp;
    % Mean
    powerTemp = readFreq * mean(readEnergy(iMemory, :)) + ...
        writeFreq * mean(writeEnergy(iMemory, :)) + ...
        refreshFreq * mean(refreshEnergy(iMemory, :)) + ...
        max(mean(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    powerrt(:,:,iMemory,3) = powerTemp;
end

%% Application space figure
% colorLimits = [...
%     1.0, 1.0, 0, 0, 0.1, 0.1;
%     1.0, 1.0, 0, 0, 0.9, 0.9;
%     0.1, 0.1, 1.0, 1.0, 0, 0;
%     0.8, 0.8, 1.0, 1.0, 0, 0;
%     0, 0, 0.1, 0.1, 1.0, 1.0;
%     0, 0, 0.9, 0.9, 1.0, 1.0;
%     0.4, 0.4, 0.4, 0.4, 0.4, 0.4;
%     0.7, 0.7, 0.7, 0.7, 0.7, 0.7
%     ];

colorLimits = [...
    1.0, 1.0, 0, 0, 0.1, 0.1;
    1.0, 1.0, 0, 0, 0.1, 0.1;
    0.1, 0.1, 1.0, 1.0, 0, 0;
    0.1, 0.1, 1.0, 1.0, 0, 0;
    0, 0, 0.1, 0.1, 1.0, 1.0;
    0, 0, 0.1, 0.1, 1.0, 1.0;
    0.1, 0.1, 0.1, 0.1, 0.1, 0.1;
    0.1, 0.1, 0.1, 0.1, 0.1, 0.1;
    ];

lineStyles = ["-";"--";"-";"--";"-";"--";"-";"--"];

dataColors = [];
for memoryNumber = 0:7
    count = sum(memoryNumbers == memoryNumber);
    if count > 0
        tmp = [linspace(colorLimits(memoryNumber + 1, 1), ...
            colorLimits(memoryNumber + 1, 2), count)', ...
            linspace(colorLimits(memoryNumber + 1, 3), ...
            colorLimits(memoryNumber + 1, 4), count)', ...
            linspace(colorLimits(memoryNumber + 1, 5), ...
            colorLimits(memoryNumber + 1, 6), count)'];
        dataColors = [dataColors; tmp];
    end
end

% Plot
figure(14)
sub_handle(2,1) = subplot(2,2,3);
hold on
for iMemory = 0:7
    dataTemp = min(powerrt(1,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
sub_handle(2,1).XScale = 'log';
sub_handle(2,1).YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
xlabel("Access rate [Hz]")
ylabel({"{\bfW/R = 0}", "Power [W]"})
grid on
ylim([1e-7, 1e-3])
sub_handle(2,1).XMinorGrid = 'off';
sub_handle(2,1).YMinorGrid = 'off';
sub_handle(2,1).YTick = 10.^[-8:-4];
sub_handle(2,1).XTick = 10.^[4:8];
box on


sub_handle(1,1) = subplot(2,2,1);
hold on
for iMemory = 0:7
    dataTemp = min(powerrt(2,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
sub_handle(1,1).XScale = 'log';
sub_handle(1,1).YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
grid on
ylim([1e-7, 1e-3])
title("RT")
ylabel({"{\bfW/R = 1}", "Power [W]"})
xticklabels(sub_handle(1,1), {})
sub_handle(1,1).XMinorGrid = 'off';
sub_handle(1,1).YMinorGrid = 'off';
sub_handle(1,1).YTick = 10.^[-8:-4];
sub_handle(1,1).XTick = 10.^[4:8];
box on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

charMemsWorkspaceNames = ["charMems_20230405194241039.mat", ...
    "charMems_20230406111705001.mat", "charMems_20230406125754577.mat"];

retTimeWorkspaceNames = ["test6_mult_20230405221043008.mat", "test6_mult_20230406082430754.mat"];

readEnergy = zeros(32,numel(charMemsWorkspaceNames));
writeEnergy = zeros(32,numel(charMemsWorkspaceNames));
refreshEnergy = zeros(32,numel(charMemsWorkspaceNames));
leakagePower = zeros(32,numel(charMemsWorkspaceNames));
latency = zeros(32,numel(charMemsWorkspaceNames));

for iMeas = 1:numel(charMemsWorkspaceNames)
    load(charMemsWorkspaceNames(iMeas))
    readEnergy(:,iMeas) = memoryCharacterisationOutputData(:,2);
    writeEnergy(:,iMeas) = memoryCharacterisationOutputData(:,3);
    refreshEnergy(:,iMeas) = memoryCharacterisationOutputData(:,4);
    leakagePower(:,iMeas) = memoryCharacterisationOutputData(:,5);
    latency(:,iMeas) = memoryCharacterisationOutputData(:,6);
end

memoryNumbers = inputData(:,1);
references = inputData(:,5);
leakagePower(memoryNumbers < 6,:) = leakagePower(memoryNumbers < 6,:) / 6;

retTime = inputData(:,6);
retTimeCount = zeros(32,1);
retTimeMu = Inf(32,1);
retTimeSigma = zeros(32,1);
    
% Add actual retention times from the more elaborate measurement script
for iRet=1:numel(retTimeWorkspaceNames)
    load(retTimeWorkspaceNames(iRet))
    for memoryIdx = 1:numel(memoryNo)
	    retTimeTemp(rowNo+1,:) = 8e-8 + 2e-8 * reshape(min(criticalDelay( ...
        memoryIdx,:,:,1,:), [], 3), [numel(rowNo), 32]);
	    NRCF = @(b) norm(sum(retTimeTemp(retTimeTemp ~= Inf)' <= retTimeTemp( ...
		    retTimeTemp ~= Inf), 2)/numel(retTimeTemp) - fcn(b, log(retTimeTemp( ...
		    retTimeTemp ~= Inf))));
	    B = fminsearch(NRCF, [-5; 1]);
        worstCellRetTime = min(retTimeTemp, [], 'all');
        
        select = and(memoryNumbers == memoryNo(memoryIdx), ...
            references == memoryReferenceVoltages(memoryIdx));
        retTime(select) = worstCellRetTime;
        retTimeCount(select) = sum(retTimeTemp == worstCellRetTime, 'all');
        retTimeMu(select) = B(1);
        retTimeSigma(select) = B(2);
        
%         fprintf("%2d: mu, sigma, min = %7.3f, %7.3f, %e (%dx)\n", ...
% 		    memoryIdx, B(1), B(2), worstCellRetTime(memoryIdx), ...
% 		    sum(retTime == min(retTime, [], 'all'), 'all'))
    end
end

% Build application space figure with min and max energy values
% Compute power over application space for each memory type
power4k = NaN([size(oF), numel(memoryNumbers), 3]);

for iMemory = 1:numel(memoryNumbers)
    refreshFreq = 1/retTime(iMemory);
    % Min
    powerTemp = readFreq * min(readEnergy(iMemory, :)) + ...
        writeFreq * min(writeEnergy(iMemory, :)) + ...
        refreshFreq * min(refreshEnergy(iMemory, :)) + ...
        max(min(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power4k(:,:,iMemory,1) = powerTemp;
    % Max
    powerTemp = readFreq * max(readEnergy(iMemory, :)) + ...
        writeFreq * max(writeEnergy(iMemory, :)) + ...
        refreshFreq * max(refreshEnergy(iMemory, :)) + ...
        max(max(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power4k(:,:,iMemory,2) = powerTemp;
    % Mean
    powerTemp = readFreq * mean(readEnergy(iMemory, :)) + ...
        writeFreq * mean(writeEnergy(iMemory, :)) + ...
        refreshFreq * mean(refreshEnergy(iMemory, :)) + ...
        max(mean(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power4k(:,:,iMemory,3) = powerTemp;
end

% Plot
sub_handle(2,2) = subplot(2,2,4);
hold on
for iMemory = 0:7
    dataTemp = min(power4k(1,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
sub_handle(2,2).XScale = 'log';
sub_handle(2,2).YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
grid on
ylim([1e-9, 1e-3])
yticklabels(sub_handle(2,2), {})
sub_handle(2,2).XMinorGrid = 'off';
sub_handle(2,2).YMinorGrid = 'off';
sub_handle(2,2).YTick = 10.^[-8:-4];
sub_handle(2,2).XTick = 10.^[4:8];
xlabel("Access rate [Hz]")
box on

sub_handle(1,2) = subplot(2,2,2);
hold on
for iMemory = 0:7
    dataTemp = min(power4k(2,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
sub_handle(1,2).XScale = 'log';
sub_handle(1,2).YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
grid on
title("4.2 K")
ylim([1e-9, 1e-3])
xticklabels(sub_handle(1,2), {})
yticklabels(sub_handle(1,2), {})
sub_handle(1,2).XMinorGrid = 'off';
sub_handle(1,2).YMinorGrid = 'off';
sub_handle(1,2).YTick = 10.^[-8:-4];
sub_handle(1,2).XTick = 10.^[4:8];
box on

linkaxes([sub_handle(2,1) sub_handle(1,1) sub_handle(2,2) sub_handle(1,2)],'xy')

% legend('2T NW-PR (SVT)', '2T NW-PR (LVT)', '3T NW-PR (SVT)',  ...
%     '3T NW-PR (LVT)', '3T PW-PR (SVT)',  '3T PW-PR (LVT)', ...
%     '6T SRAM (SVT)', '6T SRAM (LVT)', 'Location', 'southoutside', ...
%     'NumColumns', 2)

legend({'2T NW-PR', '', '3T NW-PR', '', '3T PW-PR', '', '6T SRAM', ''}, ...
    'Location', 'southoutside', 'NumColumns', 2)

%% Figure sizing
fig_handle = figure(14);
fig_width  = 8.6; %cm
fig_height = 4.5; %cm
set(fig_handle, "Units", "centimeters")
set(fig_handle, 'Position',[60 25 fig_width fig_height]);
%set(fig_handle, 'Position',[-20 0 fig_width fig_height]);
set(fig_handle.Children(1), "ItemTokenSize", [10,18])

subs_xoff = 0.15;
sub_xoff = 0;
sub_yoff = 0.17;
subs_width = 0.825;
sub_width = subs_width/2;
subs_height = 0.76;
sub_height = subs_height/2;
nrows = 2;
ncols = 2;
for row = 1:nrows
    for col = 1:ncols
        set(sub_handle(row, col), "FontName", "Times")
        set(sub_handle(row, col), "FontSize", 7)
        %set(sub_handle(row, col), "LineWidth", 0.5)
        set(sub_handle(row,col), 'Position', [(col-1)/ncols*subs_width + sub_xoff + subs_xoff, (nrows - row)/nrows*subs_height + sub_yoff, sub_width, sub_height]);
    end
end

% Legend location
legend_handle = fig_handle.Children(1);
sub_pos = sub_handle(1,1).Position;
leg_pos = legend_handle.Position;
leg_pos(1:2) = sub_pos(1:2) + (sub_pos(3) - leg_pos(3))/2;
set(legend_handle, "Position", leg_pos)
%%
fig_handle.Renderer = "painters";
% saveas(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\power_comparison_2.eps", 'epsc')