%% Plot RO delay at 300K and 4K
file_300k = "test3_20221221105853929.mat";
file_4k = "test3_20230112082926091.mat";

% Initialize figure
fig_handle = figure(1);
set(fig_handle, "Units", "centimeters")
position_f = fig_handle.Position;
set(fig_handle, "Position", [position_f(1), position_f(2), 7.5, 3.5])
set(gca, "FontName", "Times")
set(gca, "FontSize", 8)

hold on
xlimits = [0, 255];

% Plot 300K
load(file_300k)
t_delay_300k = 1e12./(2*roFrequency);
fit_300k = polyfit(roDelay, t_delay_300k, 1);

sc_300k = scatter(roDelay, t_delay_300k, 20, 'rx', 'LineWidth', 1);
ft_300k = plot(xlimits, polyval(fit_300k, xlimits), 'r--', 'LineWidth', 1);

% Plot 4K
load(file_4k)
t_delay_4k = 1e12./(2*roFrequency);
fit_4k = polyfit(roDelay, t_delay_4k, 1);

sc_4k = scatter(roDelay, t_delay_4k, 20, 'bx', 'LineWidth', 1);
ft_4k = plot(xlimits, polyval(fit_4k, xlimits), 'b--', 'LineWidth', 1);


hold off
grid on
xlim(xlimits)
xlabel("Delay setting")
ylim([0, 5e3])
ylabel("Loop delay [ps]")
legend([ft_300k, ft_4k], sprintf("300 K: %.2f ps/step", fit_300k(1)), ...
    sprintf("4.2 K: %.2f ps/step", fit_4k(1)), "Location", "best")
set(fig_handle.Children(1), "ItemTokenSize", [20,18])

% saveas(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\ro_delay.eps", 'epsc')