fig_handle = figure(2);
svtPlotMax = 256;
lvtPlotMin = 511 - svtPlotMax;

svtAddrSel = 0:255;
lvtAddrSel = 511 - svtAddrSel;

linewidth = 1.0;

foreground_color_rt = [1.0, 0.2, 0.1];
background_color_rt = [1.0, 0.85, 0.82];
foreground_linewidth_rt = linewidth;
background_linewidth_rt = linewidth;

foreground_color_4k = [0.0, 0.0, 1.0];
background_color_4k = [0.8, 0.8, 1.0];
foreground_linewidth_4k = linewidth;
background_linewidth_4k = linewidth;

%% Load RT data
load("test11_20230307073714092.mat")
load("test12_20230307082206219.mat")

% Find cell transfer by mapping source follower output to input
wl0Transfer = interp1(sfOutputVoltages, sfInputVoltages, wl0Output, "linear", "extrap");
wl1bl0Transfer = interp1(sfOutputVoltages, sfInputVoltages, wl1bl0Output, "linear", "extrap");
wl1bl1Transfer = interp1(sfOutputVoltages, sfInputVoltages, wl1bl1Output, "linear", "extrap");

% Compute distance to diagonal
xDiag = linspace(0,1.1,111);
wl0YDiag = zeros(numel(cellNo), numel(xDiag));
wl1bl0YDiag = zeros(numel(cellNo), numel(xDiag));
wl1bl1YDiag = zeros(numel(cellNo), numel(xDiag));
for cellIdx = 1:numel(cellNo)
    for xInd = 1:numel(xDiag)
        x = xDiag(xInd);
        wl0YDiag(cellIdx, xInd) = (interp1(wl0Transfer(cellIdx,:) ...
            - (1.1 - 2 * x + inputVoltages), inputVoltages, 0) - x) ...
            * sqrt(2);
        wl1bl0YDiag(cellIdx, xInd) = (interp1(wl1bl0Transfer(cellIdx,:) ...
            - (1.1 - 2 * x + inputVoltages), inputVoltages, 0) - x) ...
            * sqrt(2);
        wl1bl1YDiag(cellIdx, xInd) = (interp1(wl1bl1Transfer(cellIdx,:) ...
            - (1.1 - 2 * x + inputVoltages), inputVoltages, 0) - x) ...
            * sqrt(2);
    end
end

% Plot SNM figures (SVT)
% Hold SNM
subplot(2,3,1)
plot(wl0Transfer(ismember(cellNo, svtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_rt, 'Color', background_color_rt)
hold on
plot(inputVoltages, wl0Transfer(ismember(cellNo, svtAddrSel),:), 'LineWidth', foreground_linewidth_rt, 'Color', foreground_color_rt)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_RT(1,1) = min(max(max(wl0YDiag(ismember(cellNo, svtAddrSel),:)) - fliplr(min(...
    wl0YDiag(ismember(cellNo, svtAddrSel),:)))), max(-(max(wl0YDiag(ismember(cellNo, svtAddrSel),:)) - ...
    fliplr(min(wl0YDiag(ismember(cellNo, svtAddrSel),:))))));
title("Hold (SVT)")

% Read SNM
subplot(2,3,2)
plot(wl1bl1Transfer(ismember(cellNo, svtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_rt, 'Color', background_color_rt)
hold on
plot(inputVoltages, wl1bl1Transfer(ismember(cellNo, svtAddrSel),:), 'LineWidth', foreground_linewidth_rt, 'Color', foreground_color_rt)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_RT(1,2) = min(max(max(wl1bl1YDiag(ismember(cellNo, svtAddrSel),:)) - fliplr(min(...
    wl1bl1YDiag(ismember(cellNo, svtAddrSel),:)))), max(-(max(wl1bl1YDiag(ismember(cellNo, svtAddrSel),...
    :)) - fliplr(min(wl1bl1YDiag(ismember(cellNo, svtAddrSel),:))))));
title("Read (SVT)")

% Write SNM
subplot(2,3,3)
plot(wl1bl1Transfer(ismember(cellNo, svtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_rt, 'Color', background_color_rt)
hold on
plot(inputVoltages, wl1bl0Transfer(ismember(cellNo, svtAddrSel),:), 'LineWidth', foreground_linewidth_rt, 'Color', foreground_color_rt)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_RT(1,3) = -min(max(wl1bl0YDiag(ismember(cellNo, svtAddrSel),:)) - ...
    fliplr(min(wl1bl1YDiag(ismember(cellNo, svtAddrSel),:))));
title("Write (SVT)")

% Plot SNM figures (LVT)
% Hold SNM
subplot(2,3,4)
plot(wl0Transfer(ismember(cellNo, lvtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_rt, 'Color', background_color_rt)
hold on
plot(inputVoltages, wl0Transfer(ismember(cellNo, lvtAddrSel),:), 'LineWidth', foreground_linewidth_rt, 'Color', foreground_color_rt)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_RT(2,1) = min(max(max(wl0YDiag(ismember(cellNo, lvtAddrSel),:)) - fliplr(min(...
    wl0YDiag(ismember(cellNo, lvtAddrSel),:)))), max(-(max(wl0YDiag(ismember(cellNo, lvtAddrSel),:)) - ...
    fliplr(min(wl0YDiag(ismember(cellNo, lvtAddrSel),:))))));
title("Hold (LVT)")

% Read SNM
subplot(2,3,5)
plot(wl1bl1Transfer(ismember(cellNo, lvtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_rt, 'Color', background_color_rt)
hold on
plot(inputVoltages, wl1bl1Transfer(ismember(cellNo, lvtAddrSel),:), 'LineWidth', foreground_linewidth_rt, 'Color', foreground_color_rt)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_RT(2,2) = min(max(max(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:)) - fliplr(min(...
    wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:)))), max(-(max(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),...
    :)) - fliplr(min(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:))))));
title("Read (LVT)")

% Write SNM
subplot(2,3,6)
plot(wl1bl1Transfer(ismember(cellNo, lvtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_rt, 'Color', background_color_rt)
hold on
plot(inputVoltages, wl1bl0Transfer(ismember(cellNo, lvtAddrSel),:), 'LineWidth', foreground_linewidth_rt, 'Color', foreground_color_rt)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_RT(2,3) = -min(max(wl1bl0YDiag(ismember(cellNo, lvtAddrSel),:)) - ...
    fliplr(min(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:))));
title("Write (LVT)")

%% Load 4K data
load("test11_20230309113258275.mat")
load("test12_20230309113440655.mat")

% Find cell transfer by mapping source follower output to input
wl0Transfer = interp1(sfOutputVoltages, sfInputVoltages, wl0Output, "linear", "extrap");
wl1bl0Transfer = interp1(sfOutputVoltages, sfInputVoltages, wl1bl0Output, "linear", "extrap");
wl1bl1Transfer = interp1(sfOutputVoltages, sfInputVoltages, wl1bl1Output, "linear", "extrap");

% Compute distance to diagonal
xDiag = linspace(0,1.1,111);
wl0YDiag = zeros(numel(cellNo), numel(xDiag));
wl1bl0YDiag = zeros(numel(cellNo), numel(xDiag));
wl1bl1YDiag = zeros(numel(cellNo), numel(xDiag));
for cellIdx = 1:numel(cellNo)
    for xInd = 1:numel(xDiag)
        x = xDiag(xInd);
        wl0YDiag(cellIdx, xInd) = (interp1(wl0Transfer(cellIdx,:) ...
            - (1.1 - 2 * x + inputVoltages), inputVoltages, 0) - x) ...
            * sqrt(2);
        wl1bl0YDiag(cellIdx, xInd) = (interp1(wl1bl0Transfer(cellIdx,:) ...
            - (1.1 - 2 * x + inputVoltages), inputVoltages, 0) - x) ...
            * sqrt(2);
        wl1bl1YDiag(cellIdx, xInd) = (interp1(wl1bl1Transfer(cellIdx,:) ...
            - (1.1 - 2 * x + inputVoltages), inputVoltages, 0) - x) ...
            * sqrt(2);
    end
end

% Plot SNM figures (SVT)
% Hold SNM
subplot(2,3,1)
hold on
plot(wl0Transfer(ismember(cellNo, svtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_4k, 'Color', background_color_4k)
plot(inputVoltages, wl0Transfer(ismember(cellNo, svtAddrSel),:), 'LineWidth', foreground_linewidth_4k, 'Color', foreground_color_4k)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_4K(1,1) = min(max(max(wl0YDiag(ismember(cellNo, svtAddrSel),:)) - fliplr(min(...
    wl0YDiag(ismember(cellNo, svtAddrSel),:)))), max(-(max(wl0YDiag(ismember(cellNo, svtAddrSel),:)) - ...
    fliplr(min(wl0YDiag(ismember(cellNo, svtAddrSel),:))))));
title("Hold (SVT)")

% Read SNM
subplot(2,3,2)
hold on
plot(wl1bl1Transfer(ismember(cellNo, svtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_4k, 'Color', background_color_4k)
plot(inputVoltages, wl1bl1Transfer(ismember(cellNo, svtAddrSel),:), 'LineWidth', foreground_linewidth_4k, 'Color', foreground_color_4k)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_4K(1,2) = min(max(max(wl1bl1YDiag(ismember(cellNo, svtAddrSel),:)) - fliplr(min(...
    wl1bl1YDiag(ismember(cellNo, svtAddrSel),:)))), max(-(max(wl1bl1YDiag(ismember(cellNo, svtAddrSel),...
    :)) - fliplr(min(wl1bl1YDiag(ismember(cellNo, svtAddrSel),:))))));
title("Read (SVT)")

% Write SNM
subplot(2,3,3)
hold on
plot(wl1bl1Transfer(ismember(cellNo, svtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_4k, 'Color', background_color_4k)
plot(inputVoltages, wl1bl0Transfer(ismember(cellNo, svtAddrSel),:), 'LineWidth', foreground_linewidth_4k, 'Color', foreground_color_4k)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_4K(1,3) = -min(max(wl1bl0YDiag(ismember(cellNo, svtAddrSel),:)) - ...
    fliplr(min(wl1bl1YDiag(ismember(cellNo, svtAddrSel),:))));
title("Write (SVT)")

% Plot SNM figures (LVT)
% Hold SNM
subplot(2,3,4)
hold on
plot(wl0Transfer(ismember(cellNo, lvtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_4k, 'Color', background_color_4k)
plot(inputVoltages, wl0Transfer(ismember(cellNo, lvtAddrSel),:), 'LineWidth', foreground_linewidth_4k, 'Color', foreground_color_4k)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_4K(2,1) = min(max(max(wl0YDiag(ismember(cellNo, lvtAddrSel),:)) - fliplr(min(...
    wl0YDiag(ismember(cellNo, lvtAddrSel),:)))), max(-(max(wl0YDiag(ismember(cellNo, lvtAddrSel),:)) - ...
    fliplr(min(wl0YDiag(ismember(cellNo, lvtAddrSel),:))))));
title("Hold (LVT)")

% Read SNM
subplot(2,3,5)
hold on
plot(wl1bl1Transfer(ismember(cellNo, lvtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_4k, 'Color', background_color_4k)
plot(inputVoltages, wl1bl1Transfer(ismember(cellNo, lvtAddrSel),:), 'LineWidth', foreground_linewidth_4k, 'Color', foreground_color_4k)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_4K(2,2) = min(max(max(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:)) - fliplr(min(...
    wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:)))), max(-(max(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),...
    :)) - fliplr(min(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:))))));
title("Read (LVT)")

% Write SNM
subplot(2,3,6)
hold on
plot(wl1bl1Transfer(ismember(cellNo, lvtAddrSel),:), inputVoltages, 'LineWidth', background_linewidth_4k, 'Color', background_color_4k)
plot(inputVoltages, wl1bl0Transfer(ismember(cellNo, lvtAddrSel),:), 'LineWidth', foreground_linewidth_4k, 'Color', foreground_color_4k)
hold off
daspect([1,1,1])
xlim([0, 1.1])
ylim([0, 1.1])
SNM_4K(2,3) = -min(max(wl1bl0YDiag(ismember(cellNo, lvtAddrSel),:)) - ...
    fliplr(min(wl1bl1YDiag(ismember(cellNo, lvtAddrSel),:))));
title("Write (LVT)")

%%
nrows = 2;
ncols = 3;

for row = 1:nrows
    for col = 1:ncols
        sub_handle(row,col) = subplot(nrows,ncols,(row-1)*ncols+col);
        grid on
    end
end

%%
fig_width  = 8.6; %cm
fig_height = 6.2; %cm
sub_width  = 0.24;
sub_height = 0.3;
sub_xoff   = 0.125;
sub_yoff   = 0.142;
col_scale  = 0.9;
set(fig_handle, "Units", "centimeters")
%set(fig_handle, 'Position',[60 25 fig_width fig_height]);
set(fig_handle, 'Position',[10 5 fig_width fig_height]);
for row = 1:nrows
    for col = 1:ncols
        sub_handle(row,col).XLabel.String = "V_{in} [V]";
        if col == 1
            sub_handle(row,col).YLabel.String = "V_{out} [V]";
        end
%         set(sub_handle(row,col), 'Position', [(row-1)/nrows + sub_xoff, (ncols - col)/ncols + sub_yoff, sub_width, sub_height]);
        set(sub_handle(row,col), "FontName", "Times")
        set(sub_handle(row,col), "FontSize", 8)
        set(sub_handle(row,col), 'Position', [col_scale*(col-1)/ncols + sub_xoff, (nrows - row)/nrows + sub_yoff, sub_width, sub_height]);
    end
end

% Graph numbers/letters
label_xoff = 0.04;
label_yoff = 0.06;
for row = 1:nrows
    for col = 1:ncols
        txt = sprintf("(%c-%d)", char(96+row), col);
%         dim = [(row-1)/nrows + label_xoff, (ncols - col)/ncols + label_yoff, sub_width, 0.03];
        dim = [col_scale*(col-1)/ncols + label_xoff, (nrows - row)/nrows + label_yoff, sub_width, 0.03];
        annotation('textbox', dim, 'String', txt, "FontName", "Times", "FontSize", 9, "LineStyle", "none")
    end
end

%%
fig_handle.Renderer = "painters";
% saveas(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\sram_snm_v3.eps", 'epsc')