%% Setup figure
fig_handle = figure(1);
set(fig_handle, "Units", "centimeters")
position_f = fig_handle.Position;
set(fig_handle, "Position", [position_f(1), position_f(2), 9.0, 3.8])

%% Temp sweep
load('singleCellTemperatureRetention_20230313093940214.mat')
dataNo = 1;
dataIdx = 1;

x1 = temperature(dataIdx:numel(dataNo):(end-numel(dataNo)));
x2 = temperature((dataIdx+1):numel(dataNo):(end-numel(dataNo)+1));
xm = mean([x1; x2]);
y1 = max((retention(dataIdx,:) - 1) * 360e-9 + 300e-9, 0);
y2 = (retention(dataIdx,:)) * 360e-9 + 300e-9;
ym = y1;

pointSel = abs(x1 - x2) < 1.9;
xm = xm(pointSel);
ym = ym(pointSel);

scatter(xm, ym, "k.")
set(gca, "YScale", "log")
xlabel("Temperature [K]")
ylabel("Retention time [s]")

lowEaTemp = [80, 150];
lowEaFit = NaN(numel(dataNo), 2);
highEaTemp = [190, 270];
highEaFit = NaN(numel(dataNo), 2);

kb = 8.617333262e-5; % Boltzmann constant in [eV/K]

for dataIdx = 1:numel(dataNo)
    x1 = 1./temperature(dataIdx:numel(dataNo):(end-numel(dataNo)));
    x2 = 1./temperature((dataIdx+1):numel(dataNo):(end-numel(dataNo)+1));
    xm = 1./mean([1./x1; 1./x2]);
    y1 = log(1./max((retention(dataIdx,:) - 1) * 360e-9 + 300e-9, 0));
    y2 = log(1./(retention(dataIdx,:) * 360e-9 + 300e-9));
    ym = y2;

    x = xm(1./xm > highEaTemp(1) & 1./xm < highEaTemp(2));
    y = ym(1./xm > highEaTemp(1) & 1./xm < highEaTemp(2));
    highEaFit(dataIdx, :) = polyfit(x, y, 1);

    xm = xm(500:end);
    ym = ym(500:end);
    x = xm(1./xm > lowEaTemp(1) & 1./xm < lowEaTemp(2));
    y = ym(1./xm > lowEaTemp(1) & 1./xm < lowEaTemp(2));
    lowEaFit(dataIdx, :) = polyfit(x, y, 1);
end
hold on
for dataIdx = 1:numel(dataNo)
    t = linspace(min(temperature), max(temperature), 100);

    retentionFit = 1./(exp(highEaFit(dataIdx,:) * [1./t; ones(size(t))]));
    plot(t, retentionFit, 'r:', 'LineWidth', 0.5)
    retentionFit = 1./(exp(lowEaFit(dataIdx,:) * [1./t; ones(size(t))]));
    plot(t, retentionFit, 'r:', 'LineWidth', 0.5)

    retentionFit = 1./(...
        exp(lowEaFit(dataIdx,:) * [1./t; ones(size(t))]) + ...
        exp(highEaFit(dataIdx,:) * [1./t; ones(size(t))]));
    plot(t, retentionFit, 'r--', 'LineWidth', 1.5)
    [~, idx] = min(abs(t - min(highEaTemp)));
    text(t(idx) - 25, retentionFit(idx) / 500, sprintf("E_{a,high} = %.3f eV", ...
        -highEaFit(dataIdx, 1) * kb), 'FontName', 'Times', 'FontSize', 8)

    [~, idx] = min(abs(t - min(lowEaTemp)));
    text(t(idx) - 50, retentionFit(idx) / 5, sprintf("E_{a,low} = %.4f eV", ...
        -lowEaFit(dataIdx, 1) * kb), 'FontName', 'Times', 'FontSize', 8)
end
hold off
xlim([0 300])
ylim([5e-8, 1e-1])
grid on
ax = gca;
ax.YMinorGrid = 'off';
ax.YMinorTick = "on";
ax.YTick = 10.^(-7:-1);

load('test6_20230103134610231.mat')
retention_300K = criticalDelay(:,:,2,:,:);
retention_300K = 8e-8 + 2e-8 * retention_300K(:);
retention_300K_temp = startTemperature;

load('test6_20230310110423629.mat')
retention_4K = criticalDelay(:,:,2,:,:);
retention_4K = 8e-8 + 2e-8 * retention_4K(retention_4K < Inf);
retention_4K_temp = startTemperature;

hold on
swarmchart(retention_300K_temp * ones(size(retention_300K)), retention_300K, 'r.', 'XJitter', 'density', 'XJitterWidth', 4)
swarmchart(retention_4K_temp * ones(size(retention_4K)), retention_4K, 'b.', 'XJitter', 'density', 'XJitterWidth', 4)
hold off

legendText = {'Cell (0,0)', '', '', '2-Arrhenius Fit', 'All cells @ RT', 'All cells @ 4.2 K'};
legend(legendText, "Location", "southwest", "AutoUpdate", "off")
set(fig_handle.Children(1), "ItemTokenSize", [25,18])
set(gca, "FontName", "Times")
set(gca, "FontSize", 8)

%% Save figure
fig_handle.Renderer = "painters";
% saveas(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\2t_cell_retention_v4.eps", 'epsc')