wsPaths = [
    'singleCellTemperatureRetention_20230313154208619.mat';
    'singleCellTemperatureRetention_20230313165307138.mat';
    'singleCellTemperatureRetention_20230405095501077.mat';
    'singleCellTemperatureRetention_20230314130643921.mat'];

cellName = [
    '3T NW-PR (SVT)';
    '3T NW-PR (LVT)';
    '3T PW-PR (SVT)';
    '3T PW-PR (LVT)'];

scatterColor = {
    [1,0,0], [1,0,0];
    [0,1,0], [0,1,0];
    [0,0,1], [0,0,1];
    [0.2,0.2,0.2], [0.2, 0.2, 0.2]};

lineStyles = {'-', '--', '-.', '--'};

% cellName = {'3T NW-PR', '', '3T PW-PR', ''};
% 
% scatterColor = {
%     [0,1,0], [0,1,0];
%     [0,1,0], [0,1,0];
%     [0,0,1], [0,0,1];
%     [0,0,1], [0,0,1]};
% 
% lineStyles = {'-', '--', '-', '--'};

figure(1230)

% Low SN voltage ellipse
x1 = 40/2;
x2 = 45/2;
y1 = log10(1e-3);
y2 = log10(1e-4);
eccentricity = 0.85;
num_points = 200;
a = 1/2*sqrt((x2-x1)^2+(y2-y1)^2);
b = a*sqrt(1-eccentricity^2);
t = linspace(0,2*pi, num_points);
X = a*cos(t);
Y = b*sin(t);
w = atan2(y2-y1,x2-x1);
x_low_sn = (x1+x2)/2 + X*cos(w) - Y*sin(w);
y_low_sn = (y1+y2)/2 + X*sin(w) + Y*cos(w);
plot(2*x_low_sn, 10.^y_low_sn, 'k', "LineWidth", 0.5)

hold on

% High SN voltage ellipse
x1 = 250/50;
x2 = 270/50;
y1 = log10(2e-6);
y2 = log10(6e-6);
eccentricity = 0.95;
num_points = 200;
a = 1/2*sqrt((x2-x1)^2+(y2-y1)^2);
b = a*sqrt(1-eccentricity^2);
t = linspace(0,2*pi, num_points);
X = a*cos(t);
Y = b*sin(t);
w = atan2(y2-y1,x2-x1);
x_low_sn = (x1+x2)/2 + X*cos(w) - Y*sin(w);
y_low_sn = (y1+y2)/2 + X*sin(w) + Y*cos(w);
plot(50*x_low_sn, 10.^y_low_sn, 'k', "LineWidth", 0.5)

% legendText = string.empty(2*numel(wsPaths), 0);
legendText = string.empty(numel(wsPaths), 0);
plotH = NaN(4,1);

for cellIdx = 1:size(wsPaths,1)
    load(wsPaths(cellIdx,:))
    for dataIdx = 1:numel(dataNo)
        x1 = temperature(dataIdx:numel(dataNo):(end-numel(dataNo)));
        x2 = temperature((dataIdx+1):numel(dataNo):(end-numel(dataNo)+1));
        xm = mean([x1; x2]);
        y1 = max((retention(dataIdx,:) - 1) * 360e-9 + 300e-9, 0);
        y2 = (retention(dataIdx,:)) * 360e-9 + 300e-9;
        ym = y1;
%         selection = ym < 1;
%         selection = and(abs(x1-x2) < 3, ym < 1);
        selection = abs(x1-x2) < 3;
        xtemp = xm(selection);
        ytemp = ym(selection);
        [xtemp, itemp] = sort(xtemp);
        ytemp = ytemp(itemp);

        ysmooth = movmean(ytemp,5);
        plotH(cellIdx) = plot(xtemp, ysmooth, lineStyles{cellIdx}, 'Color', scatterColor{cellIdx, dataIdx}, 'LineWidth', 1);

%         plot(xtemp, ytemp, 'Color', scatterColor{cellIdx, dataIdx})
%         scatter(xm(selection), ym(selection), ".", "MarkerEdgeColor", scatterColor{cellIdx, dataIdx})
%         legendText(2*(cellIdx - 1) + dataIdx) = sprintf("%s: data = %d", cellName(cellIdx,:), dataNo(dataIdx));
    end 
end
text(30, 2e-2, {"Low SN", "voltage"}, "FontSize", 8, "FontName", "Times")
text(220, 3e-7, {"High SN", "voltage"}, "FontSize", 8, "FontName", "Times")
hold off
set(gca, "YScale", "log")
xlabel("Temperature [K]")
ylabel("Retention time [s]")
leg_handle = legend(plotH, cellName, "Location", "south", "AutoUpdate", "off");
% title("3T GC cells: retention time")
grid on
ylim([10^(-7.5), 10^(-1)])
ax = gca;
ax.YTick = 10.^(-7:-1);
ax.YMinorGrid = 'off';
ax.YMinorTick = 'off';

%% Figure sizing
fig_handle = figure(1230);
ax_handle = gca;
fig_width  = 8.6; %cm
fig_height = 4.3; %cm
set(fig_handle, "Units", "centimeters")
set(fig_handle, 'Position',[60 25 fig_width fig_height]);
set(ax_handle, "FontSize", 8)
set(ax_handle, "FontName", "Times")
leg_handle.Position
set(leg_handle, "Position", [0.27 0.2202 0.3787 0.3097]);

%%
fig_handle.Renderer = "painters";
%saveas(fig_handle, "H:\My Documents\CryoMemory_measurements\MATLAB\figures_jssc\3t_cell_retention_v5.eps", 'epsc')