%% Compute energy over application space
% Compute the energy for each point in the application space for each
% memory type

% Load workspace
load("H:\My Documents\CryoMemory_measurements\MATLAB\memoryCharacterisation\charMems_20230331092010113")

%% Data format
% [Retention time, read energy, write energy, refresh energy, leakage
% power, operation latency]

% Example data
% data = [...
%     300e-9, 500e-15, 150e-15, 32 * 620e-15, 0, 1e-9;
%     1e-6, 450e-15, 200e-15, 32 * 600e-15, 0, 2e-9;
%     Inf, 600e-15, 500e-15, 0, 100e-9, 0.5e-9];
data = memoryCharacterisationOutputData;

% colorLimits = [...
%     1.0, 1.0, 0, 0.7, 0.1, 0.1;
%     1.0, 1.0, 0, 0.7, 0.9, 0.9;
%     0.1, 0.1, 1.0, 1.0, 0, 0.7;
%     0.9, 0.9, 1.0, 1.0, 0, 0.7;
%     0, 0.7, 0.1, 0.1, 1.0, 1.0;
%     0, 0.7, 0.9, 0.9, 1.0, 1.0;
%     0.4, 0.4, 0.4, 0.4, 0.4, 0.4;
%     0.7, 0.7, 0.7, 0.7, 0.7, 0.7
%     ];

colorLimits = [...
    1.0, 1.0, 0, 0, 0.1, 0.1;
    1.0, 1.0, 0, 0, 0.9, 0.9;
    0.1, 0.1, 1.0, 1.0, 0, 0;
    0.9, 0.9, 1.0, 1.0, 0, 0;
    0, 0, 0.1, 0.1, 1.0, 1.0;
    0, 0, 0.9, 0.9, 1.0, 1.0;
    0.4, 0.4, 0.4, 0.4, 0.4, 0.4;
    0.7, 0.7, 0.7, 0.7, 0.7, 0.7
    ];

dataColors = [];
memoryNumbers = inputData(:,1);
for memoryNumber = 0:7
    count = sum(memoryNumbers == memoryNumber);
    if count > 0
        tmp = [linspace(colorLimits(memoryNumber + 1, 1), ...
            colorLimits(memoryNumber + 1, 2), count)', ...
            linspace(colorLimits(memoryNumber + 1, 3), ...
            colorLimits(memoryNumber + 1, 4), count)', ...
            linspace(colorLimits(memoryNumber + 1, 5), ...
            colorLimits(memoryNumber + 1, 6), count)'];
        dataColors = [dataColors; tmp];
    end
end

%% Compute power over application space for each memory type
writeFrequencyRatio = linspace(0, 0.5, 101);
operationFrequency = logspace(log10(1e3), log10(1e9), 501);

[oF, wFR] = meshgrid(operationFrequency, writeFrequencyRatio);

readFreq = oF .* (1 - wFR);
writeFreq = oF .* wFR;

power = NaN([size(oF), size(data, 1)]);

for iData = 1:size(data,1)
    refreshFreq = 1/data(iData,1);
    powerTemp = readFreq * data(iData, 2) + writeFreq * data(iData, 3) ...
        + refreshFreq * data(iData, 4) + max(data(iData, 5), 0);
    powerTemp(oF > 1/data(iData, 6)) = NaN;
    power(:,:,iData) = powerTemp;
end

%% Plot
figure(12)
t = tiledlayout(10,1);
ax1 = nexttile([6,1]);
applicationSpace = zeros(size(oF));
minPower = min(power, [], 3);
for iData = 1:size(data,1)
    applicationSpace(power(:,:,iData) == minPower) = iData;
end
hold on
for i=1:8
    fill([0,1,0], [0,0,1], colorLimits(i, 1:2:end))
end
img = imagesc(operationFrequency, writeFrequencyRatio, applicationSpace);
hold off
img.Parent.YDir = 'normal';
img.Parent.XScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
ylabel("f_W/(f_R + f_W)")
ylim([0, 0.5])
colormap(img.Parent, [1, 1, 1; dataColors])
clim([0, size(data,1)])
title("Lowest power consumption memory")
xticklabels(ax1, {})
% legend('2T NW-PR (SVT)', '2T NW-PR (LVT)', '3T NW-PR (SVT)',  ...
%     '3T NW-PR (LVT)', '3T PW-PR (SVT)',  '3T PW-PR (LVT)', ...
%     '6T SRAM (SVT)', '6T SRAM (LVT)', 'Location', 'west')

ax2 = nexttile([4,1]);
hold on
memoryNumbersUnique = unique(memoryNumbers);
memoryPowerMin = NaN(numel(operationFrequency), numel(memoryNumbersUnique));
memoryPowerMax = NaN(numel(operationFrequency), numel(memoryNumbersUnique));
for memIdx = 1:numel(memoryNumbersUnique)
	memoryNumber = memoryNumbersUnique(memIdx);
    sel = and(memoryNumbers == memoryNumber, ismember(1:numel(memoryNumbers), unique(applicationSpace(:)))');
    if any(sel)
	    memoryPowerMin(:, memIdx) = min(power(:, :, sel), [], [1,3]);
	    memoryPowerMax(:, memIdx) = max(power(:, :, sel), [], [1,3]);
	    fill([operationFrequency(~isnan(memoryPowerMin(:, memIdx))), fliplr(operationFrequency(~isnan(memoryPowerMin(:, memIdx))))], [memoryPowerMin(~isnan(memoryPowerMin(:, memIdx)), memIdx)', fliplr(memoryPowerMax(~isnan(memoryPowerMin(:, memIdx)), memIdx)')], colorLimits(memoryNumber + 1, 1:2:end), 'FaceAlpha', 0.3, 'LineStyle', 'none')
    end
end
for memIdx = 1:numel(memoryNumbersUnique)
	memoryNumber = memoryNumbersUnique(memIdx);
    sel = and(memoryNumbers == memoryNumber, ismember(1:numel(memoryNumbers), unique(applicationSpace(:)))');
    if any(sel)
	    plot(operationFrequency, memoryPowerMin(:, memIdx), 'Color', colorLimits(memoryNumber + 1, 2:2:end))
	    plot(operationFrequency, memoryPowerMax(:, memIdx), 'Color', colorLimits(memoryNumber + 1, 1:2:end))
    end
end
hold off
ax2.XScale = 'log';
ax2.YScale = 'log';
xlabel("f_R + f_W [Hz]")
ylabel("Power [W]")
xlim([min(operationFrequency), max(operationFrequency)])
grid on
ylim([1e-9, 1e-3])
ax2.YTick = [1e-8, 1e-7, 1e-6, 1e-5, 1e-4];

t.TileSpacing = 'none';
linkaxes([ax1 ax2],'x')