charMemsWorkspaceNames = ["charMems_20230406172956365.mat", ...
    "charMems_20230407111710500.mat", "charMems_20230407124013212.mat"];

retTimeWorkspaceNames = ["test6_mult_20230411090830355.mat"];

readEnergy = zeros(32,numel(charMemsWorkspaceNames));
writeEnergy = zeros(32,numel(charMemsWorkspaceNames));
refreshEnergy = zeros(32,numel(charMemsWorkspaceNames));
leakagePower = zeros(32,numel(charMemsWorkspaceNames));
latency = zeros(32,numel(charMemsWorkspaceNames));

for iMeas = 1:numel(charMemsWorkspaceNames)
    load(charMemsWorkspaceNames(iMeas))
    readEnergy(:,iMeas) = memoryCharacterisationOutputData(:,2);
    writeEnergy(:,iMeas) = memoryCharacterisationOutputData(:,3);
    refreshEnergy(:,iMeas) = memoryCharacterisationOutputData(:,4);
    leakagePower(:,iMeas) = memoryCharacterisationOutputData(:,5);
    latency(:,iMeas) = memoryCharacterisationOutputData(:,6);
end

% RO speedup correction (latency is using the 4K RO speed instead of RT)
latency = (latency - 180e-12)/17*20 + 180e-12;

memoryNumbers = inputData(:,1);
references = inputData(:,5);
leakagePower(memoryNumbers < 6,:) = leakagePower(memoryNumbers < 6,:) / 6;

retTime = inputData(:,6);
retTimeCount = zeros(32,1);
retTimeMu = Inf(32,1);
retTimeSigma = zeros(32,1);
    
% Add actual retention times from the more elaborate measurement script
for iRet=1:numel(retTimeWorkspaceNames)
    load(retTimeWorkspaceNames(iRet))
    for memoryIdx = 1:numel(memoryNo)
	    retTimeTemp(rowNo+1,:) = 8e-8 + 2e-8 * reshape(min(criticalDelay( ...
        memoryIdx,:,:,1,:), [], 3), [numel(rowNo), 32]);
	    NRCF = @(b) norm(sum(retTimeTemp(retTimeTemp ~= Inf)' <= retTimeTemp( ...
		    retTimeTemp ~= Inf), 2)/numel(retTimeTemp) - fcn(b, log(retTimeTemp( ...
		    retTimeTemp ~= Inf))));
	    B = fminsearch(NRCF, [-5; 1]);
        worstCellRetTime = min(retTimeTemp, [], 'all');
        
        select = and(memoryNumbers == memoryNo(memoryIdx), ...
            references == memoryReferenceVoltages(memoryIdx));
        retTime(select) = worstCellRetTime;
        retTimeCount(select) = sum(retTimeTemp == worstCellRetTime, 'all');
        retTimeMu(select) = B(1);
        retTimeSigma(select) = B(2);
        
%         fprintf("%2d: mu, sigma, min = %7.3f, %7.3f, %e (%dx)\n", ...
% 		    memoryIdx, B(1), B(2), worstCellRetTime(memoryIdx), ...
% 		    sum(retTime == min(retTime, [], 'all'), 'all'))
    end
end

% Build application space figure with min and max energy values
% Compute power over application space for each memory type
writeFrequencyRatio = linspace(0, 0.5, 101);
operationFrequency = logspace(log10(1e3), log10(1e9), 501);

[oF, wFR] = meshgrid(operationFrequency, writeFrequencyRatio);

readFreq = oF .* (1 - wFR);
writeFreq = oF .* wFR;

power = NaN([size(oF), numel(memoryNumbers), 3]);

for iMemory = 1:numel(memoryNumbers)
    refreshFreq = 1/retTime(iMemory);
    % Min
    powerTemp = readFreq * min(readEnergy(iMemory, :)) + ...
        writeFreq * min(writeEnergy(iMemory, :)) + ...
        refreshFreq * min(refreshEnergy(iMemory, :)) + ...
        max(min(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,1) = powerTemp;
    % Max
    powerTemp = readFreq * max(readEnergy(iMemory, :)) + ...
        writeFreq * max(writeEnergy(iMemory, :)) + ...
        refreshFreq * max(refreshEnergy(iMemory, :)) + ...
        max(max(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,2) = powerTemp;
    % Mean
    powerTemp = readFreq * mean(readEnergy(iMemory, :)) + ...
        writeFreq * mean(writeEnergy(iMemory, :)) + ...
        refreshFreq * mean(refreshEnergy(iMemory, :)) + ...
        max(mean(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power(:,:,iMemory,3) = powerTemp;
end

selPower = zeros(numel(memoryNumbers), 1);
minPower = min(power(:,:,:,3), [], 3);
for iMemory = 1:numel(memoryNumbers)
    memPower = power(:,:,iMemory, 3);
    selPower(iMemory) = any(memPower == minPower, 'all');
end

selMemoryWise = zeros(numel(memoryNumbers), 1);
% for iMemory = 1:numel(memoryNumbers)
%     memPower = power(:,:,iMemory, 3);
%     memNumber = memoryNumbers(iMemory);
%     minPower = min(power(:,:,memoryNumbers == memNumber,3), [], 3);
%     selMemoryWise(iMemory) = any(memPower == minPower, 'all');
% end
for iMemory = 1:numel(memoryNumbers)
    if ~any(memoryNumbers(selPower == 1) == memoryNumbers(iMemory))
        %memPower = power(:,:,iMemory, 3);
        memNumber = memoryNumbers(iMemory);
        %minPower = min(power(:,:,memoryNumbers == memNumber,3), [], 3);
        %selMemoryWise(iMemory) = any(memPower == minPower, 'all');

        % Max retention time
        selMemoryWise(iMemory) = (retTime(iMemory) == ...
            max(retTime(memoryNumbers == memNumber)));        
    end
end

% Select only the memory configurations that have the lowest power 
% somewhere for each memory type
selection = and(or(selPower, selMemoryWise), ~any(latency > 3e-9, 2));

%% Application space figure
colorLimits = [...
    1.0, 1.0, 0, 0, 0.1, 0.1;
    1.0, 1.0, 0, 0, 0.9, 0.9;
    0.1, 0.1, 1.0, 1.0, 0, 0;
    0.8, 0.8, 1.0, 1.0, 0, 0;
    0, 0, 0.1, 0.1, 1.0, 1.0;
    0, 0, 0.9, 0.9, 1.0, 1.0;
    0.4, 0.4, 0.4, 0.4, 0.4, 0.4;
    0.7, 0.7, 0.7, 0.7, 0.7, 0.7
    ];

dataColors = [];
for memoryNumber = 0:7
    count = sum(memoryNumbers == memoryNumber);
    if count > 0
        tmp = [linspace(colorLimits(memoryNumber + 1, 1), ...
            colorLimits(memoryNumber + 1, 2), count)', ...
            linspace(colorLimits(memoryNumber + 1, 3), ...
            colorLimits(memoryNumber + 1, 4), count)', ...
            linspace(colorLimits(memoryNumber + 1, 5), ...
            colorLimits(memoryNumber + 1, 6), count)'];
        dataColors = [dataColors; tmp];
    end
end

% Plot
figure(12)
t = tiledlayout(3,2);
ax1 = nexttile(1);
applicationSpace = zeros(size(oF));
minPower = min(power(:,:,selection,3), [], 3);
for iMemory = 1:numel(memoryNumbers)
    applicationSpace(power(:,:,iMemory,3) == minPower) = iMemory;
end
hold on
for i=1:8
    fill([0,1,0], [0,0,1], colorLimits(i, 1:2:end))
end
img = imagesc(operationFrequency, writeFrequencyRatio, applicationSpace);
hold off
img.Parent.YDir = 'normal';
img.Parent.XScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
ylabel("f_W/(f_R + f_W)")
ylim([0, 0.5])
ax1.YTick = [0,0.25,0.5];
colormap(img.Parent, [1, 1, 1; dataColors])
clim([0, size(data,1)])
title("RT")
xticklabels(ax1, {})
% legend('2T NW-PR (SVT)', '2T NW-PR (LVT)', '3T NW-PR (SVT)',  ...
%     '3T NW-PR (LVT)', '3T PW-PR (SVT)',  '3T PW-PR (LVT)', ...
%     '6T SRAM (SVT)', '6T SRAM (LVT)', 'Location', 'west')

ax2 = nexttile(3);
hold on
memoryNumbersUnique = unique(memoryNumbers);
memoryPowerMin = NaN(numel(operationFrequency), numel(memoryNumbersUnique));
memoryPowerMax = NaN(numel(operationFrequency), numel(memoryNumbersUnique));
for memIdx = 1:numel(memoryNumbersUnique)
	memoryNumber = memoryNumbersUnique(memIdx);
    sel = and(memoryNumbers == memoryNumber, selection);
    if any(sel)
	    memoryPowerMin(:, memIdx) = min(power(:, :, sel, 3), [], [1,3]);
	    memoryPowerMax(:, memIdx) = max(power(:, :, sel, 3), [], [1,3]);
	    fill([operationFrequency(~isnan(memoryPowerMin(:, memIdx))), ...
            fliplr(operationFrequency(~isnan(memoryPowerMin(:, memIdx))))], ...
            [memoryPowerMin(~isnan(memoryPowerMin(:, memIdx)), memIdx)', ...
            fliplr(memoryPowerMax(~isnan(memoryPowerMin(:, memIdx)), memIdx)')], ...
            colorLimits(memoryNumber + 1, 1:2:end), 'FaceAlpha', 0.7, 'LineStyle', 'none')
	    plot(operationFrequency, memoryPowerMin(:, memIdx), 'Color', colorLimits(memoryNumber + 1, 2:2:end))
	    plot(operationFrequency, memoryPowerMax(:, memIdx), 'Color', colorLimits(memoryNumber + 1, 1:2:end))
    end
end
hold off
ax2.XScale = 'log';
ax2.YScale = 'log';
xlabel("f_R + f_W [Hz]")
ylabel("Power [W]")
xlim([min(operationFrequency), max(operationFrequency)])
grid on
ylim([1e-8, 1e-3])
ax2.YTick = [1e-8, 1e-7, 1e-6, 1e-5, 1e-4];
ax2.YMinorGrid = 'off';
ax2.YMinorTick = 'off';
ax2.XTick = [1e4, 1e6, 1e8];
ax2.XMinorGrid = 'off';

t.TileSpacing = 'none';
linkaxes([ax1 ax2],'x')

%% Print table
for memoryNo = 0:7
    fprintf("----------------------------------------------------------\n")
    fprintf("Memory %d\n", memoryNo)
    % Retention time
    fprintf("Retention time:\n")
    mu = retTimeMu(and(selection, memoryNumbers == memoryNo));
    fprintf("Mu: %.3f - %.3f\n", min(mu), max(mu))
    sigma = retTimeSigma(and(selection, memoryNumbers == memoryNo));
    fprintf("Sigma: %.3f - %.3f\n", min(sigma), max(sigma))
    worstTime = retTime(and(selection, memoryNumbers == memoryNo));
    worstAmount = retTimeCount(and(selection, memoryNumbers == memoryNo));
    [~,iMin] = min(worstTime);
    [~,iMax] = max(worstTime);
    fprintf("Worst: %e (%dx) - %e (%dx)\n", min(worstTime), ...
        worstAmount(iMin), max(worstTime), worstAmount(iMax))
    % Power
    fprintf("Energy/power:\n")
    Er = readEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Eread: %.3f - %.3f fJ/op\n", 1e15 * min(Er, [], "all"), ...
        1e15 * max(Er, [], "all"))
    Ew = writeEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Ewrite: %.3f - %.3f fJ/op\n", 1e15 * min(Ew, [], "all"), ...
        1e15 * max(Ew, [], "all"))
    Eref = refreshEnergy(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Erefresh: %.3f - %.3f pJ\n", 1e12 * min(Eref, [], "all"), ...
        1e12 * max(Eref, [], "all"))
    Tlat = latency(and(selection, memoryNumbers == memoryNo), :);
    fprintf("Latency: %.3f - %.3f ns\n", 1e9 * min(Tlat, [], "all"), ...
        1e9 * max(Tlat, [], "all"))
end