charMemsWorkspaceNames = ["charMems_20230406172956365.mat", ...
    "charMems_20230407111710500.mat", "charMems_20230407124013212.mat"];

retTimeWorkspaceNames = ["test6_mult_20230411090830355.mat"];

readEnergy = zeros(32,numel(charMemsWorkspaceNames));
writeEnergy = zeros(32,numel(charMemsWorkspaceNames));
refreshEnergy = zeros(32,numel(charMemsWorkspaceNames));
leakagePower = zeros(32,numel(charMemsWorkspaceNames));
latency = zeros(32,numel(charMemsWorkspaceNames));

for iMeas = 1:numel(charMemsWorkspaceNames)
    load(charMemsWorkspaceNames(iMeas))
    readEnergy(:,iMeas) = memoryCharacterisationOutputData(:,2);
    writeEnergy(:,iMeas) = memoryCharacterisationOutputData(:,3);
    refreshEnergy(:,iMeas) = memoryCharacterisationOutputData(:,4);
    leakagePower(:,iMeas) = memoryCharacterisationOutputData(:,5);
    latency(:,iMeas) = memoryCharacterisationOutputData(:,6);
end

% RO speedup correction (latency is using the 4K RO speed instead of RT)
latency = (latency - 180e-12)/17*20 + 180e-12;

memoryNumbers = inputData(:,1);
references = inputData(:,5);
leakagePower(memoryNumbers < 6,:) = leakagePower(memoryNumbers < 6,:) / 6;

retTime = inputData(:,6);
retTimeCount = zeros(32,1);
retTimeMu = Inf(32,1);
retTimeSigma = zeros(32,1);
    
% Add actual retention times from the more elaborate measurement script
for iRet=1:numel(retTimeWorkspaceNames)
    load(retTimeWorkspaceNames(iRet))
    for memoryIdx = 1:numel(memoryNo)
	    retTimeTemp(rowNo+1,:) = 8e-8 + 2e-8 * reshape(min(criticalDelay( ...
        memoryIdx,:,:,1,:), [], 3), [numel(rowNo), 32]);
	    NRCF = @(b) norm(sum(retTimeTemp(retTimeTemp ~= Inf)' <= retTimeTemp( ...
		    retTimeTemp ~= Inf), 2)/numel(retTimeTemp) - fcn(b, log(retTimeTemp( ...
		    retTimeTemp ~= Inf))));
	    B = fminsearch(NRCF, [-5; 1]);
        worstCellRetTime = min(retTimeTemp, [], 'all');
        
        select = and(memoryNumbers == memoryNo(memoryIdx), ...
            references == memoryReferenceVoltages(memoryIdx));
        retTime(select) = worstCellRetTime;
        retTimeCount(select) = sum(retTimeTemp == worstCellRetTime, 'all');
        retTimeMu(select) = B(1);
        retTimeSigma(select) = B(2);
        
%         fprintf("%2d: mu, sigma, min = %7.3f, %7.3f, %e (%dx)\n", ...
% 		    memoryIdx, B(1), B(2), worstCellRetTime(memoryIdx), ...
% 		    sum(retTime == min(retTime, [], 'all'), 'all'))
    end
end

% Build application space figure with min and max energy values
% Compute power over application space for each memory type
writeReadRatio = linspace(0, 1, 2);
operationFrequency = logspace(log10(1e3), log10(1e9), 501);

[oF, wRR] = meshgrid(operationFrequency, writeReadRatio);

readFreq = oF ./ (1 + wRR);
writeFreq = oF - readFreq;

powerrt = NaN([size(oF), numel(memoryNumbers), 3]);

for iMemory = 1:numel(memoryNumbers)
    refreshFreq = 1/retTime(iMemory);
    % Min
    powerTemp = readFreq * min(readEnergy(iMemory, :)) + ...
        writeFreq * min(writeEnergy(iMemory, :)) + ...
        refreshFreq * min(refreshEnergy(iMemory, :)) + ...
        max(min(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    powerrt(:,:,iMemory,1) = powerTemp;
    % Max
    powerTemp = readFreq * max(readEnergy(iMemory, :)) + ...
        writeFreq * max(writeEnergy(iMemory, :)) + ...
        refreshFreq * max(refreshEnergy(iMemory, :)) + ...
        max(max(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    powerrt(:,:,iMemory,2) = powerTemp;
    % Mean
    powerTemp = readFreq * mean(readEnergy(iMemory, :)) + ...
        writeFreq * mean(writeEnergy(iMemory, :)) + ...
        refreshFreq * mean(refreshEnergy(iMemory, :)) + ...
        max(mean(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    powerrt(:,:,iMemory,3) = powerTemp;
end

%% Application space figure
% colorLimits = [...
%     1.0, 1.0, 0, 0, 0.1, 0.1;
%     1.0, 1.0, 0, 0, 0.9, 0.9;
%     0.1, 0.1, 1.0, 1.0, 0, 0;
%     0.8, 0.8, 1.0, 1.0, 0, 0;
%     0, 0, 0.1, 0.1, 1.0, 1.0;
%     0, 0, 0.9, 0.9, 1.0, 1.0;
%     0.4, 0.4, 0.4, 0.4, 0.4, 0.4;
%     0.7, 0.7, 0.7, 0.7, 0.7, 0.7
%     ];

colorLimits = [...
    1.0, 1.0, 0, 0, 0.1, 0.1;
    1.0, 1.0, 0, 0, 0.1, 0.1;
    0.1, 0.1, 1.0, 1.0, 0, 0;
    0.1, 0.1, 1.0, 1.0, 0, 0;
    0, 0, 0.1, 0.1, 1.0, 1.0;
    0, 0, 0.1, 0.1, 1.0, 1.0;
    0.1, 0.1, 0.1, 0.1, 0.1, 0.1;
    0.1, 0.1, 0.1, 0.1, 0.1, 0.1;
    ];

lineStyles = ["-";"--";"-";"--";"-";"--";"-";"--"];

dataColors = [];
for memoryNumber = 0:7
    count = sum(memoryNumbers == memoryNumber);
    if count > 0
        tmp = [linspace(colorLimits(memoryNumber + 1, 1), ...
            colorLimits(memoryNumber + 1, 2), count)', ...
            linspace(colorLimits(memoryNumber + 1, 3), ...
            colorLimits(memoryNumber + 1, 4), count)', ...
            linspace(colorLimits(memoryNumber + 1, 5), ...
            colorLimits(memoryNumber + 1, 6), count)'];
        dataColors = [dataColors; tmp];
    end
end

% Plot
figure(14)
t = tiledlayout(3,2);
ax1 = nexttile(1);
hold on
for iMemory = 0:7
    dataTemp = min(powerrt(1,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
ax1.XScale = 'log';
ax1.YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
ylabel({"{\bfRT}", "Power [W]"})
title("W/R = 0")
xticklabels(ax1, {})
grid on
ylim([1e-7, 1e-3])
ax1.XMinorGrid = 'off';
ax1.YMinorGrid = 'off';
ax1.YTick = 10.^[-8:-4];
ax1.XTick = 10.^[4:8];


ax2 = nexttile(2);
hold on
for iMemory = 0:7
    dataTemp = min(powerrt(2,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
ax2.XScale = 'log';
ax2.YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
grid on
ylim([1e-7, 1e-3])
title("W/R = 1")
xticklabels(ax2, {})
yticklabels(ax2, {})
ax2.XMinorGrid = 'off';
ax2.YMinorGrid = 'off';
ax2.YTick = 10.^[-8:-4];
ax2.XTick = 10.^[4:8];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

charMemsWorkspaceNames = ["charMems_20230405194241039.mat", ...
    "charMems_20230406111705001.mat", "charMems_20230406125754577.mat"];

retTimeWorkspaceNames = ["test6_mult_20230405221043008.mat", "test6_mult_20230406082430754.mat"];

readEnergy = zeros(32,numel(charMemsWorkspaceNames));
writeEnergy = zeros(32,numel(charMemsWorkspaceNames));
refreshEnergy = zeros(32,numel(charMemsWorkspaceNames));
leakagePower = zeros(32,numel(charMemsWorkspaceNames));
latency = zeros(32,numel(charMemsWorkspaceNames));

for iMeas = 1:numel(charMemsWorkspaceNames)
    load(charMemsWorkspaceNames(iMeas))
    readEnergy(:,iMeas) = memoryCharacterisationOutputData(:,2);
    writeEnergy(:,iMeas) = memoryCharacterisationOutputData(:,3);
    refreshEnergy(:,iMeas) = memoryCharacterisationOutputData(:,4);
    leakagePower(:,iMeas) = memoryCharacterisationOutputData(:,5);
    latency(:,iMeas) = memoryCharacterisationOutputData(:,6);
end

memoryNumbers = inputData(:,1);
references = inputData(:,5);
leakagePower(memoryNumbers < 6,:) = leakagePower(memoryNumbers < 6,:) / 6;

retTime = inputData(:,6);
retTimeCount = zeros(32,1);
retTimeMu = Inf(32,1);
retTimeSigma = zeros(32,1);
    
% Add actual retention times from the more elaborate measurement script
for iRet=1:numel(retTimeWorkspaceNames)
    load(retTimeWorkspaceNames(iRet))
    for memoryIdx = 1:numel(memoryNo)
	    retTimeTemp(rowNo+1,:) = 8e-8 + 2e-8 * reshape(min(criticalDelay( ...
        memoryIdx,:,:,1,:), [], 3), [numel(rowNo), 32]);
	    NRCF = @(b) norm(sum(retTimeTemp(retTimeTemp ~= Inf)' <= retTimeTemp( ...
		    retTimeTemp ~= Inf), 2)/numel(retTimeTemp) - fcn(b, log(retTimeTemp( ...
		    retTimeTemp ~= Inf))));
	    B = fminsearch(NRCF, [-5; 1]);
        worstCellRetTime = min(retTimeTemp, [], 'all');
        
        select = and(memoryNumbers == memoryNo(memoryIdx), ...
            references == memoryReferenceVoltages(memoryIdx));
        retTime(select) = worstCellRetTime;
        retTimeCount(select) = sum(retTimeTemp == worstCellRetTime, 'all');
        retTimeMu(select) = B(1);
        retTimeSigma(select) = B(2);
        
%         fprintf("%2d: mu, sigma, min = %7.3f, %7.3f, %e (%dx)\n", ...
% 		    memoryIdx, B(1), B(2), worstCellRetTime(memoryIdx), ...
% 		    sum(retTime == min(retTime, [], 'all'), 'all'))
    end
end

% Build application space figure with min and max energy values
% Compute power over application space for each memory type
power4k = NaN([size(oF), numel(memoryNumbers), 3]);

for iMemory = 1:numel(memoryNumbers)
    refreshFreq = 1/retTime(iMemory);
    % Min
    powerTemp = readFreq * min(readEnergy(iMemory, :)) + ...
        writeFreq * min(writeEnergy(iMemory, :)) + ...
        refreshFreq * min(refreshEnergy(iMemory, :)) + ...
        max(min(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power4k(:,:,iMemory,1) = powerTemp;
    % Max
    powerTemp = readFreq * max(readEnergy(iMemory, :)) + ...
        writeFreq * max(writeEnergy(iMemory, :)) + ...
        refreshFreq * max(refreshEnergy(iMemory, :)) + ...
        max(max(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power4k(:,:,iMemory,2) = powerTemp;
    % Mean
    powerTemp = readFreq * mean(readEnergy(iMemory, :)) + ...
        writeFreq * mean(writeEnergy(iMemory, :)) + ...
        refreshFreq * mean(refreshEnergy(iMemory, :)) + ...
        max(mean(leakagePower(iMemory, :)), 0);
    powerTemp(oF > 1/min(latency(iMemory, :))) = NaN;
    power4k(:,:,iMemory,3) = powerTemp;
end

% Plot
ax3 = nexttile(3);
hold on
for iMemory = 0:7
    dataTemp = min(power4k(1,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
ax3.XScale = 'log';
ax3.YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
ylabel({"{\bf4.2 K}", "Power [W]"})
grid on
ylim([1e-9, 1e-3])
ax3.XMinorGrid = 'off';
ax3.YMinorGrid = 'off';
ax3.YTick = 10.^[-8:-4];
ax3.XTick = 10.^[4:8];
xlabel("Access rate [Hz]")

ax4 = nexttile(4);
hold on
for iMemory = 0:7
    dataTemp = min(power4k(2,:,memoryNumbers == iMemory,3), [], 3);
    plot(operationFrequency, dataTemp, lineStyles(iMemory+1), 'Color', colorLimits(iMemory + 1, 1:2:end))
end
hold off
ax4.XScale = 'log';
ax4.YScale = 'log';
xlim([min(operationFrequency), max(operationFrequency)])
grid on
ylim([1e-9, 1e-3])
yticklabels(ax4, {})
ax4.XMinorGrid = 'off';
ax4.YMinorGrid = 'off';
ax4.YTick = 10.^[-8:-4];
ax4.XTick = 10.^[4:8];
xlabel("Access rate [Hz]")

t.TileSpacing = 'none';
linkaxes([ax1 ax2 ax3 ax4],'xy')

% legend('2T NW-PR (SVT)', '2T NW-PR (LVT)', '3T NW-PR (SVT)',  ...
%     '3T NW-PR (LVT)', '3T PW-PR (SVT)',  '3T PW-PR (LVT)', ...
%     '6T SRAM (SVT)', '6T SRAM (LVT)', 'Location', 'southoutside', ...
%     'NumColumns', 2)

legend({'2T NW-PR', '', '3T NW-PR', '', '3T PW-PR', '', '6T SRAM', ''}, ...
    'Location', 'southoutside', 'NumColumns', 2)