;;; Cell retention fast reset
; Measure the retention time of a single cell in a write-read-read pattern
; while resetting the SAs between each read.

; Memory settings register
; R4 = settings register address
; R5 = settings register data
; R9 = reset settings register data
init $r4 0b0000100000
init $r5 0b00011010000010001000000011111111
init $r9 0b00011110000010001000000011111111

; Row of interest and data
; R6 = address of row of interest
; R7 = data (opposite of bitline background set in settings register)
init $r6 0b0000000000
init $r7 0x00000000

; Infinite loop broken by read_stop
; R1 = loop counter
init $r1 0

; Retention time check
; R2 = read result
; R3 = shift left amount (= 31 - columnNumber)
; R8 = expected data (single bit of r7)
init $r2 0
init $r3 31
shri $r7 31 $r8

; Write data
write $r7 $r6

; Loop start
; Reset SAs
write $r9 $r4
read $r6 $r0
; Read data
write $r5 $r4
read $r6 $r2
; Check if cell has failed
shl $r2 $r3 $r2
shri $r2 31 $r2
beq $r2 $r8 2
beq $r0 $r0 0

; Loop end (jump back)
addi $r1 1 $r1
beq $r0 $r0 -9