;;; Loop random write
; Loop write instructions with random data and addresses for write power 
; measurements

; Memory settings register
; R4 = settings register address
; R5 = settings register data, all settings halfway, SA test mode disabled,
; bitline background enabled and set to 1 (for write 0; set 0 for write 1)
init $r4 0b0000100000
init $r5 0b00011010000010001000000011111111
write $r5 $r4

; PRNG
; Xorshift PRNG (32b example from https://en.wikipedia.org/wiki/Xorshift)
; Update routine in C:
; x = x ^ (x << 13);
; x = x ^ (x >> 17);
; x = x ^ (x << 5);
; -> 12 instructions to update both address and data 
; (Note: will never visit all 0 word)
; R1 = data PRNG
; R2 = address PRNG
; R3 = storage for shifted versions
init $r1 0xFAB105EB
init $r2 0xA571A203

; Address calculation
; Memory base address + 5-bit part of the address PRNG
; R6 = base address
; R7 = write address
init $r6 = 0b0000000000

; Loop: 1 write instruction every 21 cycles
; Calculate address and write
; Take upper 5 bits from address PRNG: R7 = R2 >> (32 - 5)
shri $r2 27 $r7
; Add base address: R7 = R6 + R7
add $r6 $r7 $r7
; Write data to combined address: write R1 to R7
write $r1 $r7

; Update PRNG
shli $r1 13 $r3
xor $r1 $r3 $r1
shri $r1 17 $r3
xor $r1 $r3 $r1
shli $r1 5 $r3
xor $r1 $r3 $r1
shli $r2 13 $r3
xor $r2 $r3 $r2
shri $r2 17 $r3
xor $r2 $r3 $r2
shli $r2 5 $r3
xor $r2 $r3 $r2

; Loop
beq $r0 $r0 -15