;;; Fast row retention assembly file
; Perform write + repeated read operations to determine the first failing 
; cell and an estimate of its retention time

; Memory settings register
; R4 = settings register address
; R5 = settings register data, all settings halfway, SA test mode disabled,
; bitline background enabled and set to 1 (for write 0; set 0 for write 1)
; R8 = reset settings register data
init $r4 0b0000100000
init $r5 0b00011010000010001000000011111111
init $r8 0b00011110000010001000000011111111

; Row of interest and data
; R6 = address of row of interest
; R7 = data (opposite of bitline background set in settings register)
; R9 = read reference (filter out the cells with 0 retention time)
; R10 = R7 XOR R9: if 0xFFFFFFFF (R31), all cells failed instantly 
; --> retention time of 0
init $r6 0b0000000000
init $r7 0x00000000
init $r9 0
init $r10 0
init $r31 0xFFFFFFFF

; Infinite loop broken by read_stop
; R1 = current wait loop duration
; R2 = wait loop counter
; R11 = number of repeats
; R12 = repeat counter
init $r1 0
init $r2 0
init $r11 0
init $r12 0

; Reset SAs
write $r8 $r4
read $r6 $r0
write $r5 $r4
; Write data
write $r7 $r6
; Read for reference
read $r6 $r9
; Check if there is at least 1 working cell
xor $r9 $r7 $r10
bneq $r10 $r31 2
beq $r0 $r0 0

; Loop start
add $r0 $r0 $r12
; Number of repeats loop start
add $r0 $r0 $r2
; Reset SAs
write $r8 $r4
read $r6 $r0
write $r5 $r4
; Write data
write $r7 $r6
; Wait for R1 loops (#cycles = 8 + 2 * $r1)
addi $r2 1 $r2
bltq $r2 $r1 -1
; Read data
read_stop $r7 $r6 $r0
; Number of repeats loop end (R11 = 0 and R11 = 1 are both 1 run)
addi $r12 1 $r12
blt $r12 $r11 -9

; Loop end (jump back)
addi $r1 1 $r1
beq $r0 $r0 -12