%% Test 10: Characterise Row Hammer
% This script is used to characterise row hammer on the dynamic cells.

runTime = string(datetime('now', 'Format', 'yyyyMMddHHmmssSSS'));
testFolder = "C:\Users\radamsteegt\Documents\MATLAB\tests\" + ...
    "10_test_row_hammer\";
workspaceName = sprintf("%stest10_%s", testFolder, runTime);

%% Run startup script
% Connect with the FPGA and instruments and set up the initial instrument 
% output values and configurations.

if ~exist("MSI", "var")
    MSI = MeasurementSetupInterface();
end
startTemperature = MSI.getTemperature();
logFileName = MSI.LogHandle.FileName;

%% Run row hammer programs
% Run program which performs a write + read on a row and hammers
% (alternating writes) another row, once on the neighbouring row and once
% on a row far away.

% Memories to test on
memoryNo = 0;
% Victim row number
victimRow = 15;
victimRowData = 0x00000000;
% Aggressor row numbers
aggressorRows = [16,31];
aggressorLoopCount = [512, 512];
aggressorData = [0x00000000, 0xFFFFFFFF;
    0x00000000, 0xFFFFFFFF];
% Reference voltages for each memory (SRAM value doesn't matter)
memoryReferenceVoltages = [0.12, 0.15, 0.17, 0.2, 0.95, 0.95, 1.1, 1.1];
% Settings register values the same for SVT/LVT
settingsRegister = kron([ ...
    bin2dec("000"+"000"+"100000"+"1010"+"00111100"+"11111111"),...
    bin2dec("000"+"000"+"100000"+"1000"+"10000000"+"11111111"), ...
    bin2dec("000"+"000"+"100000"+"1000"+"10000000"+"11111111")], [1,1]);

% Allocate result matrices
errorCounters = zeros(numel(memoryNo), numel(aggressorRows), 32);

% Load starting program
inputProgram = Program("test_10");
inputProgram.fromAsmFile(testFolder + "test_10.asm")

for memoryIdx = 1:numel(memoryNo)
    % Set memory reference voltage
    MSI.setSupplyChannelVoltageSafe(MSI.SUPPLY_CHANNEL_REF, ...
        memoryReferenceVoltages(memoryNo(memoryIdx) + 1))

    inputProgram.Register(4) = 64 * memoryNo(memoryIdx) + 32;
    inputProgram.Register(5) = settingsRegister(memoryNo(memoryIdx) + 1);
    
    inputProgram.Register(1) = 64 * memoryNo(memoryIdx) + victimRow;
    inputProgram.Register(2) = victimRowData;

    for aggressorIdx = 1:numel(aggressorRows)
        inputProgram.Register(3) = 64 * memoryNo(memoryIdx) + ...
            aggressorRows(aggressorIdx);
        inputProgram.Register(6) = aggressorData(aggressorIdx, 1);
        inputProgram.Register(7) = aggressorData(aggressorIdx, 2);
        inputProgram.Register(8) = aggressorLoopCount(aggressorIdx);

        outputProgram = MSI.runProgram(inputProgram);

        errorCounters(memoryIdx, aggressorIdx, :) = ...
            outputProgram.ErrorCounter(:);
    end
end

%% Plot error counter heatmap of the victim rows
figure(100)
heatmap(memoryNo, aggressorRows, errorCounters(:,:,victimRow))
xlabel("Memory")
ylabel("Agressor row")
title("Victim row error counter")

%% Save data
% Save the measurement data from the active workspace into a .mat file in
% the test directory.

%save(workspaceName, "startTemperature", "logFileName", "errorCounters", ...
%    "victimRow", "victimRowData", "memoryNo", "aggressorLoopCount", ...
%    "aggressorRows", "aggressorData")