function distance = getShiftSpeedErrors(fpgaObj, shiftSpeed, binString, ...
    register)
    arguments (Input)
        fpgaObj (1,1) FpgaInterface
        shiftSpeed (1,1) uint16
        binString (1,1) string
        register (1,1) double {mustBeMember(register, [0, 1])}
    end
    arguments (Output)
        distance (1,1) double
    end
    %GETSHIFTSPEEDERRORS Get number of errors occuring when shifting data 
    %at specified speed
    %   Find the number of errors occuring when shifting the data at the
    %   specified speed by doing the following:
    %   - Set the shift speed
    %   - Shift the data through the shift register
    %   - Compare the result with the original binary string and find the
    %   number of differences (e.g. Levenshtein or Hamming distance)
    %   - Calculate average for several runs to remove the effect of
    %   (un)lucky runs

    NUM_RUNS = 10;
    dists = zeros(1, NUM_RUNS);
    
    fpgaObj.setReset(1)
    fpgaObj.setShiftSpeed(shiftSpeed)
    registerCorrect = false;
    for iRun = 1:NUM_RUNS
        fprintf("Reg %d: Speed = %d, Run = %d\n", register, shiftSpeed, ...
            iRun)

        % Only rewrite to register if it is corrupted to increase test
        % speed
        if ~registerCorrect
            fpgaObj.writeRegister(register, binString)
        end

        fpgaObj.shift(register)
        fpgaObj.shift(register)
        result = fpgaObj.readRegister(register);

        % Levenshtein distance (very slow for long strings!)
        %dists(iRun) = editDistance(result, binString);
        % Hamming distance (very fast)
        dists(iRun) = sum(char(result) ~= char(binString));

        if dists(iRun) == 0
            registerCorrect = true;
        else
            registerCorrect = false;
        end

        if (dists(iRun) > 0 && shiftSpeed == uint16(Inf))
            fprintf("Top: good, bottom: errors\n%s\n%s\n", binString, ...
                result)
            error("Errors at lowest shift speed!")
        end
    end

    distance = mean(dists);
end

