%% Test 2: Test Processor Instruction
% This script is used to test the processor instructions to verify that the
% ALU works as expected and the registers properly capture the data.

runTime = string(datetime('now', 'Format', 'yyyyMMddHHmmssSSS'));
testFolder = "C:\Users\radamsteegt\Documents\MATLAB\tests\" + ...
    "2_test_processor\";
workspaceName = sprintf("%stest2_%s", testFolder, runTime);

%% Run startup script
% Connect with the FPGA and instruments and set up the initial instrument 
% output values and configurations.

if ~exist("MSI", "var")
    MSI = MeasurementSetupInterface();
end
startTemperature = MSI.getTemperature();
logFileName = MSI.LogHandle.FileName;

%% Load a simple program
% Run a simple program that computes several additions, XORs, and shifts in
% both the register and immediate formats.

inputProgram = Program("test2");
inputProgram.fromAsmFile(testFolder + "test_2.asm")

% Run program
outputProgram = MSI.runProgram(inputProgram);

%% Check correctness
% Check if the results are as expected.

assert(outputProgram.Register(3) == 4294901930)
assert(outputProgram.Register(6) == 95550187)
assert(outputProgram.Register(9) == 976)
assert(outputProgram.Register(12) == 15)
assert(outputProgram.Register(13) == 175)
assert(outputProgram.Register(14) == 4205905376)
assert(outputProgram.Register(15) == 1073741824)
assert(outputProgram.Register(16) == 2005)

fprintf("No errors detected!\n")

%% Save data
% Save the measurement data from the active workspace into a .mat file in
% the test directory.

save(workspaceName, "startTemperature", "logFileName", "inputProgram", ...
    "outputProgram")
