;;; Test 4 assembly file
; Test the various memories for correct functionality at high speeds.
; Time between write and read is much shorter than retention time.

; Set memory settings register
; R4 = settings register address
; R5 = settings register data, all settings halfway
init $r4 0b0000100000
init $r5 0b00000010000010001000000010000000
write $r5 $r4

; Address range (start and end addresses)
; R1 = start address
; R2 = stop address
init $r1 0b0000000000
init $r2 0b0000011111

;;; Loop over the address range
; R3 = address loop variable
; Load start address
add $r0 $r1 $r3
; Loop start (jump here if $r3 <= $r2)
; Write data to $r3 and immediately read it (compare, accumulate, discard)
write $r0 $r3
read_acc $r0 $r3 $r0
; Increment address by 1
addi $r3 1 $r3
; Check if still less than or equal to stop address
; If true, jump back to start of the loop
bltq $r3 $r2 -3

; Same loop, data is all-1
init $r6 0xFFFFFFFF
add $r0 $r1 $r3
write $r6 $r3
read_acc $r6 $r3 $r0
addi $r3 1 $r3
bltq $r3 $r2 -3

; Same loop, data is alternating 0/1 with 0 LSB
init $r7 0xAAAAAAAA
add $r0 $r1 $r3
write $r7 $r3
read_acc $r7 $r3 $r0
addi $r3 1 $r3
bltq $r3 $r2 -3

; Same loop, data is alternating 0/1 with 1 LSB
init $r8 0x55555555
add $r0 $r1 $r3
write $r8 $r3
read_acc $r8 $r3 $r0
addi $r3 1 $r3
bltq $r3 $r2 -3

; Same loop, data is all-0 except middle
init $r9 0x00010000
add $r0 $r1 $r3
write $r9 $r3
read_acc $r9 $r3 $r0
addi $r3 1 $r3
bltq $r3 $r2 -3

; Same loop, data is all-1 except middle
init $r10 0xFFFF7FFF
add $r0 $r1 $r3
write $r10 $r3
read_acc $r10 $r3 $r0
addi $r3 1 $r3
bltq $r3 $r2 -3