%% Test 5: Characterise Sense Amplifiers
% This script is used to characterise the sense amplifiers of all the
% memories.

runTime = string(datetime('now', 'Format', 'yyyyMMddHHmmssSSS'));
testFolder = "C:\Users\radamsteegt\Documents\MATLAB\tests\" + ...
    "5_characterise_sense_amplifiers\";
workspaceName = sprintf("%stest5_sweep_fixedpol_%s", testFolder, runTime);

%% Run startup script
% Connect with the FPGA and instruments and set up the initial instrument 
% output values and configurations.

if ~exist("MSI", "var")
    MSI = MeasurementSetupInterface();
end
startTemperature = MSI.getTemperature();
logFileName = MSI.LogHandle.FileName;

%%

ref = 0.1;
%tref = kron(0.095, ones(1,400));
%tref = kron(ones(1,10), 0.08:0.001:0.12);
%tref = kron(ones(1,5), [0, kron(0.095, ones(1,5)), 0.3, kron(0.095, ones(1,5))]);
%tref = kron(ones(1,2), [(0.165:0.002:0.245), (0.245:-0.002:0.165)]);
tref = kron(ones(1,5), [(0.1:0.002:0.14), (0.14:-0.002:0.1)]);
%tref = zeros(1,210) + 0.07;
%tref(2:2:end) = kron(ones(1,5), (0.09:0.001:0.11));
nRuns = numel(tref);

MSI.setSupplyChannelVoltageSafe(MSI.SUPPLY_CHANNEL_TREF, tref(1))
MSI.setSupplyChannelVoltageSafe(MSI.SUPPLY_CHANNEL_REF, ref)

reference = NaN(1, nRuns);
testReference = NaN(1, nRuns);
errorCounters = NaN(32, nRuns);

memoryNo = 0;

settingsDataDisabled = bin2dec("000"+"000"+"100000"+"1000"+"01100000"+ ...
    "11111111");
settingsDataEnabled = bin2dec("000"+"001"+"100000"+"1011"+"00110110"+ ...
    "11111111");
settingsAddr = 64 * memoryNo + 32;
memoryRow = 64 * memoryNo;
saPolarity = 0xFFFFFFFF;

% Load input program
inputProgram = Program("test_5");
inputProgram.fromAsmFile(testFolder + "test_5_singleSApolarity.asm")

inputProgram.Register(4) = settingsAddr;
inputProgram.Register(5) = settingsDataDisabled;
inputProgram.Register(6) = settingsDataEnabled;
inputProgram.Register(3) = memoryRow;
inputProgram.Register(7) = saPolarity;

tic
for i = 1:nRuns
    MSI.setSupplyChannelVoltageSafe(MSI.SUPPLY_CHANNEL_TREF, tref(i))

    outputProgram = MSI.runProgram(inputProgram);

    reference(1,i) = ...
        MSI.measureSupplyChannelVoltage(MSI.SUPPLY_CHANNEL_REF);
    testReference(1,i) = ...
        MSI.measureSupplyChannelVoltage(MSI.SUPPLY_CHANNEL_TREF);
    errorCounters(:,i) = outputProgram.ErrorCounter(:);
end
toc

%% Plot
figure(500+figCounter)
plot3(reference - testReference, errorCounters(1,:), 1:numel(reference))
%hold on
%for i = 2:32
%    plot3(reference - testReference, errorCounters(i,:), 1:numel(reference))
%end
%hold off
grid on
figCounter = figCounter + 1;

%% Save data
% Save the measurement data from the active workspace into a .mat file in
% the test directory.

save(workspaceName, "startTemperature", "logFileName", "errorCounters",...
    "reference", "testReference", "saPolarity", "settingsDataEnabled", ...
    "settingsDataDisabled", "memoryRow")