;;; Test 6 assembly file
; Find the simple retention time of a row from a dynamic cell memory

; Memory settings register
; R4 = settings register address
; R5 = settings register data, all settings halfway, SA test mode disabled,
; bitline background enabled and set to 1 (for write 0; set 0 for write 1)
init $r4 0b0000100000
init $r5 0b00011010000010001000000011111111
write $r5 $r4

; Row of interest and data
; R6 = address of row of interest
; R7 = data (opposite of bitline background set in settings register)
init $r6 0b0000000000
init $r7 0x00000000

; Nested loop (31x32 + 31 = 1023 measurements)
; Increase wait delay in the inner loop and outer loop
; - If inner loop step size = 0 -> 32x the same setting for 32 settings
; - If inner loop step size = outer loop step size -> 1024 different delays
; R1 = inner loop step size
; R2 = outer loop step size
; R3 = delay amount (init with start value)
init $r1 32
init $r2 32
init $r3 0
; Loop variables
; R27 = constant 31 for comparison
; R28 = delay counter
; R29 = outer loop counter
; R30 = inner loop counter
; R31 = constant 32 for comparison
init $r27 31
init $r28 0
init $r29 0
init $r30 0
init $r31 32

; Outer loop start
add $r0 $r0 $r30
; Inner loop start
add $r0 $r0 $r28

; Write data
write $r7 $r6
; Wait
addi $r28 1 $r28
bltq $r28 $r3 -1
; Read data
read_acc $r7 $r6 $r0

; Increment delay amount by inner loop step size
add $r3 $r1 $r3
; Inner loop end
addi $r30 1 $r30
beq $r29 $r27 2
blt $r30 $r31 -8
; If above is false (R30 >= 32), below (R30 >= 31) automatically also false
blt $r30 $r27 -9

; Increment delay amount by outer loop step size
add $r3 $r2 $r3
; Outer loop end
addi $r29 1 $r29
blt $r29 $r31 -13

; Stop execution
beq $r0 $r0 0