%% Test 6: Cell Retention Time (simple)
% This script is used to find the simple retention time (fixed reference
% voltage) for all dynamic cells.

runTime = string(datetime('now', 'Format', 'yyyyMMddHHmmssSSS'));
testFolder = "C:\Users\radamsteegt\Documents\MATLAB\tests\" + ...
    "6_cell_retention_time_simple\";
workspaceName = sprintf("%stest6_%s", testFolder, runTime);

%% Run startup script
% Connect with the FPGA and instruments and set up the initial instrument 
% output values and configurations.

if ~exist("MSI", "var")
    MSI = MeasurementSetupInterface();
end
startTemperature = MSI.getTemperature();
logFileName = MSI.LogHandle.FileName;

%% Set reference voltages and perform measurements
% Set the reference voltage, loop over the memories and rows, and sweep the
% hold time between write and read instruction to find the time to failure
% for each cell.

% Reference voltages (dynamic memories only)
memoryNo = 4;
%memoryReferenceVoltages = [0.12, 0.15, 0.085, 0.2, 0.95, 0.72];
memoryReferenceVoltages = [0.085, 0.175, 0.404, 0.1, 0.977, 0.940];
memoryReferenceVoltages = [0.127, 0.319, 0.204, 0.324, 0.970, 0.931];
% Settings register values
settingsRegister = [ ...
    bin2dec("000"+"110"+"001010"+"1010"+"00110010"+"11111111"), ...
    bin2dec("000"+"110"+"001010"+"1010"+"01100100"+"11111111"), ...
    bin2dec("000"+"110"+"001111"+"0100"+"00101000"+"11111111"), ...
    bin2dec("000"+"110"+"100000"+"0100"+"00011110"+"11111111"), ...
    bin2dec("000"+"110"+"011110"+"1111"+"00111100"+"11111111"), ...
    bin2dec("000"+"110"+"011110"+"1111"+"00110010"+"11111111")];
settingsRegister = [ ...
    bin2dec("000"+"110"+"001010"+"1010"+"01111000"+"11111111"), ...
    bin2dec("000"+"110"+"001010"+"1010"+"01100100"+"11111111"), ...
    bin2dec("000"+"110"+"001111"+"0100"+"00101101"+"11111111"), ...
    bin2dec("000"+"110"+"001111"+"0100"+"00101101"+"11111111"), ...
    bin2dec("000"+"110"+"011110"+"1111"+"10110100"+"11111111"), ...
    bin2dec("000"+"110"+"011110"+"1111"+"10110100"+"11111111")];
settingsRegisterReset = bin2dec("000"+"111"+"011110"+"1111"+"10000000"+ ...
    "11111111");
clear testReferenceVoltage
% Test ref voltages for reset to 1 (for data = 0)
testReferenceVoltage(:,1) = max(memoryReferenceVoltages - 0.15, 0);
% Test ref voltages for reset to 0 (for data = 1)
testReferenceVoltage(:,2) = min(memoryReferenceVoltages + 0.15, 1.1);

% Rows of interest (select single row for anomalies or script testing)
rowNo = 0:31;
% Data of interest (can select single value)
writeData = 0:1;

% Delay settings for first program iteration
firstRunDelayInitial = 0;
firstRunDelayStep = 1000;

% Load starting program
% inputProgram = Program("test_6");
% inputProgram.fromAsmFile(testFolder + "test_6.asm")
inputProgram = Program("test_6_rst");
inputProgram.fromAsmFile(testFolder + "test_6_rst.asm")

% Allocate result matrices (every memory, every row, every data polarity, 
% 2 runs, 32 cells per run)
errorCounters = zeros(numel(memoryNo), numel(rowNo), numel(writeData), ...
    3, 32);
secondRunDelay = zeros(numel(memoryNo), numel(rowNo), numel(writeData), 2);
thirdRunDelay = zeros(numel(memoryNo), numel(rowNo), numel(writeData), 2);

% Loop over memories
for memoryIdx = 1:numel(memoryNo)
    fprintf("Memory %d/%d (= %d)\n", memoryIdx, numel(memoryNo), ...
        memoryNo(memoryIdx))
    % Load memory settings
    inputProgram.Register(4) = 64 * memoryNo(memoryIdx) + 32;
    inputProgram.Register(5) = settingsRegister(memoryNo(memoryIdx) + 1);
    inputProgram.Register(9) = settingsRegisterReset;

    % Set memory reference voltage
    MSI.setSupplyChannelVoltageSafe(MSI.SUPPLY_CHANNEL_REF, ...
        memoryReferenceVoltages(memoryNo(memoryIdx) + 1))

    % Loop over rows of interest
    for rowIdx = 1:numel(rowNo)
        fprintf("Row %d/%d (= %d)\n", rowIdx, numel(rowNo), rowNo(rowIdx))
        inputProgram.Register(6) = 64 * memoryNo(memoryIdx) + ...
            rowNo(rowIdx);
        % Loop over write data of interest
        for writeDataIdx = 1:numel(writeData)
            fprintf("Data %d/%d (= %d)\n", writeDataIdx, ...
                numel(writeData), writeData(writeDataIdx))
            % Set test reference for reset
            MSI.setSupplyChannelVoltageSafe(MSI.SUPPLY_CHANNEL_TREF, ...
                testReferenceVoltage(memoryNo(memoryIdx) + 1, ...
                writeData(writeDataIdx) + 1))

            inputProgram.Register(5) = bitset(inputProgram.Register(5), ...
                29, 1 - writeData(writeDataIdx));
            inputProgram.Register(7) = 0xFFFFFFFF * ...
                writeData(writeDataIdx);
            % Set the loop step sizes and initial value
            % Run program 3x:
            % 1. Large steps to find the point where cells start failing
            % 2. Small steps to find the point where cells start failing
            % 3. Sweep over critical range (32 x 32)

            % ----- 1 -----
            inputProgram.Register(1) = firstRunDelayStep;
            inputProgram.Register(2) = firstRunDelayStep;
            inputProgram.Register(3) = firstRunDelayInitial;

            outputProgram = MSI.runProgram(inputProgram);
            
            % Save results
            errorCounters(memoryIdx, rowIdx, writeDataIdx, 1, :) = ...
                outputProgram.ErrorCounter(:);

            % Calculate step size for the second program iteration
            delay = firstRunDelayInitial + (0:1022) * firstRunDelayStep;

            [worstErrorCount, worstCell] = max(outputProgram...
                .ErrorCounter(:));
            if worstErrorCount == 1023
                warning("Cell (%d,%d) always fails, decrease the " + ...
                    "initial delay", rowNo(rowIdx), worstCell - 1)
                worstErrorCount = max(outputProgram.ErrorCounter( ...
                    outputProgram.ErrorCounter < 1023));
                if isempty(worstErrorCount)
                    warning("All cells in row %d fail", rowNo(rowIdx))
                    continue
                end
            end
            secondRunDelay(memoryIdx, rowIdx, writeDataIdx, 1) = ...
                delay(1023 - worstErrorCount);

            [bestErrorCount, bestCell] = min(outputProgram...
                .ErrorCounter(:));
            if bestErrorCount == 0
                warning("Cell (%d,%d) always works, increase the " + ...
                    "initial delay or step size", rowNo(rowIdx), ...
                    bestCell - 1)
                bestErrorCount = min(outputProgram.ErrorCounter( ...
                    outputProgram.ErrorCounter > 0));
                if isempty(bestErrorCount)
                    warning("All cells in row %d work", rowNo(rowIdx))
                    continue
                end
            end
            secondRunDelay(memoryIdx, rowIdx, writeDataIdx, 2) = ...
                ceil((delay(1024 - bestErrorCount) - ...
                secondRunDelay(memoryIdx, rowIdx, writeDataIdx, 1)) ...
                / 1023);

            % ----- 2 -----
            inputProgram.Register(1) = secondRunDelay(memoryIdx, ...
                rowIdx, writeDataIdx, 2);
            inputProgram.Register(2) = secondRunDelay(memoryIdx, ...
                rowIdx, writeDataIdx, 2);
            inputProgram.Register(3) = secondRunDelay(memoryIdx, ...
                rowIdx, writeDataIdx, 1);

            outputProgram = MSI.runProgram(inputProgram);
            
            % Save results
            errorCounters(memoryIdx, rowIdx, writeDataIdx, 2, :) = ...
                outputProgram.ErrorCounter(:);

            % Calculate step size for the second program iteration
            worstErrorCount = max(outputProgram.ErrorCounter(:));
            bestErrorCount = min(outputProgram.ErrorCounter( ...
                outputProgram.ErrorCounter > 0));

            if ~any(outputProgram.ErrorCounter > 0)
                warning("No second run errors in row %d", rowNo(rowIdx))
                thirdRunDelay(memoryIdx, rowIdx, writeDataIdx, :) = ...
                    secondRunDelay(memoryIdx, rowIdx, writeDataIdx, :);
            else
                delay = secondRunDelay(memoryIdx, rowIdx, writeDataIdx, ...
                    1) + (-1:1023) * secondRunDelay(memoryIdx, rowIdx, ...
                    writeDataIdx, 2);
    
                thirdRunDelay(memoryIdx, rowIdx, writeDataIdx, 1) = ...
                    delay(1024 - worstErrorCount);
                thirdRunDelay(memoryIdx, rowIdx, writeDataIdx, 2) = ...
                    ceil((delay(1025 - bestErrorCount) - ...
                    thirdRunDelay(memoryIdx, rowIdx, writeDataIdx, 1))/32);
            end

            % ----- 3 -----
            inputProgram.Register(1) = 0;
            inputProgram.Register(2) = thirdRunDelay(memoryIdx, ...
                rowIdx, writeDataIdx, 2);
            inputProgram.Register(3) = thirdRunDelay(memoryIdx, ...
                rowIdx, writeDataIdx, 1);

            outputProgram = MSI.runProgram(inputProgram);
            
            % Save results
            errorCounters(memoryIdx, rowIdx, writeDataIdx, 3, :) = ...
                outputProgram.ErrorCounter(:);
            
        end
    end
end


%% Construct cell retention time heatmap
% Use the results from the measurements to construct a heatmap showing the
% best and worst cells in terms of worst case retention time.

% Assumption: cells suddenly start to fail
% - First N groups of 32: no failure (error counter + 0)
% - N+1 th group: failing sometimes (error counter + 1 to 32)
% - N+2 to 32th group: full failure (error counter + 32)
% For every element in errorCounters(:,:,:,3,:) -> find delay of N+1th
% group

delayGroup = 32 - ceil(errorCounters(:,:,:,3,:) / 32);
criticalDelay = thirdRunDelay(:,:,:,1) + thirdRunDelay(:,:,:,2) .* ...
    delayGroup;
criticalDelay(errorCounters(:,:,:,1,:) == 0) = Inf;
criticalDelay(errorCounters(:,:,:,1,:) == 1023) = 0;

for memoryIdx = 1:numel(memoryNo)
    % Plot heatmap
    figure(61 + memoryNo(memoryIdx))
    plotData = NaN(32);
    plotData(rowNo+1,:) = reshape(min(criticalDelay(memoryIdx,:,:,1,:), ...
        [], 3), [numel(rowNo), 32]);
    
    heatmap(0:31, 0:31, (8 + 2 * plotData)*1e-8)
    xlabel("Column")
    ylabel("Row")
    title(sprintf("Retention time [s] (ref = %.3f V)", ...
        memoryReferenceVoltages(memoryNo(memoryIdx) + 1)))
    R = linspace(1,0,200);
    G = 1 - R;
    B = zeros(size(R));
    % Gradient mixing gamma correction: gamma = 2.2
    colormap([R(:), G(:), B(:)].^(1/2.2))
    set(gca, 'ColorScaling', 'log')
end

%% Save data
% Save the measurement data from the active workspace into a .mat file in
% the test directory.

save(workspaceName, "startTemperature", "logFileName", "errorCounters",...
    "secondRunDelay", "thirdRunDelay", "delayGroup", "criticalDelay", ...
    "memoryNo", "rowNo", "writeData", "memoryReferenceVoltages", ...
    "testReferenceVoltage", "settingsRegisterReset")
