;;; Test 8 assembly file
; Perform 1023 write-hold-read operations to determine the average read 
; result

; Memory settings register
; R4 = settings register address
; R5 = settings register data, all settings halfway, SA test mode disabled,
; bitline background enabled and set to 1 (for write 0; set 0 for write 1)
; R9 = write duration setting (0-63)
; R11 = max write duration setting (= 63)
init $r4 0b0000100000
init $r5 0b00011010000010001000000011111111
init $r9 50
init $r11 63
; Shift R9 left by 20 positions and XOR with R5 to generate the settings
; register data
; !! R5 BITS 20-25 MUST BE ALL 0 !!
shli $r9 20 $r9
xor $r5 $r9 $r5
; R12 keeps settings register with the max write duration setting
shli $r11 20 $r11
xor $r5 $r11 $r12

; Row of interest and data
; R6 = address of row of interest
; R7 = data (opposite of bitline background set in settings register)
; R10 = reset write data (same as bitline background)
init $r6 0b0000000000
init $r7 0x00000000
init $r10 0xFFFFFFFF

; 1023x measurement loop
; R1 = loop variable
; R2 = loop limit (constant 1023)
init $r1 0
init $r2 1023

; Wait loop
; R3 = wait loop variable
; R8 = wait loop limit (constant)
init $r3 0
init $r8 0

; Loop start
add $r0 $r0 $r3
; Set write setting to max and write the background data
write $12 $r4
write $r10 $r6
; Set write setting to desired value and write data
write $r5 $r4
write $r7 $r6

; Wait
addi $r3 1 $r3
bltq $r3 $r8 -1
; Read data
; Compare with 0 instead of original data to get the SA comparison output
read_acc $r0 $r6 $r0

; Loop end
addi $r1 1 $r1
blt $r1 $r2 -9

; Stop
beq $r0 $r0 0