function [raw, corePower, controlPower] = test_9_measure_power(MSI, ...
    smuChannel, coreRelayChannel, controlRelayChannel)
    %TEST_9_MEASURE_POWER Measure the core and control power
    %   Detailed explanation goes here
    
    % Number of leakage measurements at each point
    numLeakage = 4;
    % Number of power consumption measurements at each point
    numConsumption = 8;

    raw = zeros(1, 2 * numConsumption + 4 * numLeakage);

    % Select core relay
    MSI.setRelay(coreRelayChannel)
    % Core leakage measurements
    MSI.memoryReset(1)
    pause(0.1)
    for measIdx = 1:numLeakage
        raw(0 * numConsumption + 0 * numLeakage + measIdx) = ...
            MSI.getSmuPower(smuChannel);
    end
    % Core power consumption measurements
    MSI.memoryReset(0)
    pause(0.1)
    for measIdx = 1:numConsumption
        raw(0 * numConsumption + 1 * numLeakage + measIdx) = ...
            MSI.getSmuPower(smuChannel);
    end
    % Core leakage measurements
    MSI.memoryReset(1)
    pause(0.1)
    for measIdx = 1:numLeakage
        raw(1 * numConsumption + 1 * numLeakage + measIdx) = ...
            MSI.getSmuPower(smuChannel);
    end

    % Deselect core relay
    MSI.resetRelay(coreRelayChannel)
    
%     % Select control relay
%     MSI.setRelay(controlRelayChannel)
%     % Control leakage measurements
%     MSI.memoryReset(1)
%     pause(0.1)
%     for measIdx = 1:numLeakage
%         raw(1 * numConsumption + 2 * numLeakage + measIdx) = ...
%             MSI.getSmuPower(smuChannel);
%     end
%     % Control power consumption measurements
%     MSI.memoryReset(0)
%     pause(0.1)
%     for measIdx = 1:numConsumption
%         raw(1 * numConsumption + 3 * numLeakage + measIdx) = ...
%             MSI.getSmuPower(smuChannel);
%     end
%     % Control leakage measurements
%     MSI.memoryReset(1)
%     pause(0.1)
%     for measIdx = 1:numLeakage
%         raw(2 * numConsumption + 3 * numLeakage + measIdx) = ...
%             MSI.getSmuPower(smuChannel);
%     end
%     % Deselect control relay
%     MSI.resetRelay(controlRelayChannel)
    
    % Compute single core/control power figures
    corePower = ...
        mean(raw((numLeakage + 1):(numLeakage + numConsumption))) - ...
        mean(raw([1:numLeakage, (numLeakage + numConsumption + 1):(2 * ...
        numLeakage + numConsumption)]));
    controlPower = 0;
%     controlPower = ...
%         mean(raw((3 * numLeakage + numConsumption + 1):(3 * numLeakage ...
%         + 2 * numConsumption))) - mean(raw([(2 * numLeakage + ...
%         numConsumption + 1):(3 * numLeakage + numConsumption), (3 * ...
%         numLeakage + 2 * numConsumption + 1):(4 * numLeakage + 2 * ...
%         numConsumption)]));
end

